@@ -9,9 +9,15 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-0 <> 0 # Python 3.x not supported! Use 2.6+ instead.
-
+import sys
 import os
+
+if sys.version_info[0] != 2:
+    try:
+        os.execvp("python2", ["python"] + sys.argv)
+    except OSError:
+        pass
+
 import signal
 import sys
 import tempfile
@@ -51,7 +57,7 @@ def main():
     for backend in [config.get("player", "backend"), "nullbe"]:
         try:
             player = quodlibet.init_backend(backend, app.librarian)
-        except PlayerError, error:
+        except PlayerError as error:
             print_e("%s. %s" % (error.short_desc, error.long_desc))
         else:
             break
@@ -64,34 +70,6 @@ def main():
 
     from quodlibet.qltk.songlist import SongList
 
-    try:
-        ratings = config.getint("settings", "ratings")
-    except (ValueError, TypeError):
-        pass
-    else:
-        util.RATING_PRECISION = 1.0 / ratings
-
-    try:
-        default_rating = config.getfloat("settings", "default_rating")
-    except (ValueError, TypeError):
-        pass
-    else:
-        const.DEFAULT_RATING = default_rating
-
-    try:
-        symbol = config.get("settings", "rating_symbol_full").decode("utf-8")
-    except UnicodeDecodeError:
-        pass
-    else:
-        util.RATING_SYMBOL = symbol
-
-    try:
-        symbol = config.get("settings", "rating_symbol_blank").decode("utf-8")
-    except UnicodeDecodeError:
-        pass
-    else:
-        util.RATING_SYMBOL_BLANK = symbol
-
     from quodlibet.util.collection import Album
     try:
         cover_size = config.getint("browsers", "cover_size")
@@ -121,8 +99,15 @@ def main():
     if hasattr(player, "init_plugins"):
         player.init_plugins()
 
+    from quodlibet.qltk import unity
+    unity.init("quodlibet.desktop", player)
+
     from quodlibet.qltk.songsmenu import SongsMenu
     SongsMenu.init_plugins()
+    from quodlibet.util.cover.manager import cover_plugins
+    cover_plugins.init_plugins()
+    from quodlibet.plugins.playlist import PLAYLIST_HANDLER
+    PLAYLIST_HANDLER.init_plugins()
 
     from quodlibet.qltk.quodlibetwindow import QuodLibetWindow
     app.window = window = QuodLibetWindow(library, player)
@@ -152,6 +137,11 @@ def main():
     if play:
         player.paused = False
 
+    # restore browser windows
+    from quodlibet.qltk.browser import LibraryBrowser
+    from gi.repository import GLib
+    GLib.idle_add(LibraryBrowser.restore, library, priority=GLib.PRIORITY_HIGH)
+
     quodlibet.main(window)
 
     print_d("Shutting down player device %r." % player.version_info)
@@ -174,7 +164,7 @@ def print_fifo(command):
         try:
             os.unlink(filename)
             # mkfifo fails if the file exists, so this is safe.
-            os.mkfifo(filename, 0600)
+            os.mkfifo(filename, 0o600)
 
             signal.signal(signal.SIGALRM, lambda: "" + 2)
             signal.alarm(1)
@@ -256,7 +246,7 @@ def control(c):
     from quodlibet import const
 
     if not isrunning():
-        quodlibet.exit(_("Quod Libet is not running."))
+        quodlibet.exit(_("Quod Libet is not running."), notify_startup=True)
     else:
         try:
             # This is a total abuse of Python! Hooray!
@@ -273,9 +263,9 @@ def control(c):
             except OSError:
                 pass
             if c != 'focus':
-                raise quodlibet.exit(True)
+                raise quodlibet.exit(True, notify_startup=True)
         else:
-            quodlibet.exit()
+            quodlibet.exit(notify_startup=True)
 
 
 def process_arguments():
@@ -317,6 +307,7 @@ def process_arguments():
         ("focus", _("Focus the running player")),
         ("unfilter", _("Remove active browser filters")),
         ("refresh", _("Refresh and rescan library")),
+        ("list-browsers", _("List available browsers")),
         ("print-playlist", _("Print the current playlist")),
         ("print-queue", _("Print the contents of the queue")),
         ("no-plugins", _("Start without plugins")),
@@ -397,7 +388,7 @@ def process_arguments():
             if command in validators and not validators[command](arg):
                 print_e(_("Invalid argument for '%s'.") % command)
                 print_e(_("Try %s --help.") % sys.argv[0])
-                quodlibet.exit(True)
+                quodlibet.exit(True, notify_startup=True)
             else:
                 control(command + " " + arg)
         elif command == "status":
@@ -406,6 +397,8 @@ def process_arguments():
             print_fifo("dump-playlist")
         elif command == "print-queue":
             print_fifo("dump-queue")
+        elif command == "list-browsers":
+            print_fifo("dump-browsers")
         elif command == "volume-up":
             control("volume +")
         elif command == "volume-down":
@@ -422,7 +415,7 @@ def process_arguments():
             try:
                 filename = URI(arg).filename
             except ValueError:
-                filename = os.path.abspath(util.expanduser(arg))
+                filename = os.path.abspath(util.path.expanduser(arg))
             if os.path.isdir(filename):
                 control("add-directory " + filename)
             else:
