@@ -27,7 +27,7 @@ from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk.msg import ErrorMessage
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.views import AllTreeView
-from quodlibet.qltk.x import ScrolledWindow, Alignment
+from quodlibet.qltk.x import ScrolledWindow, Alignment, Button
 
 
 FEEDS = os.path.join(const.USERDIR, "feeds")
@@ -227,6 +227,7 @@ class AudioFeeds(Browser, Gtk.VBox):
 
     def pack(self, songpane):
         container = qltk.RHPaned()
+        self.show()
         container.pack1(self, True, False)
         container.pack2(songpane, True, False)
         return container
@@ -288,8 +289,7 @@ class AudioFeeds(Browser, Gtk.VBox):
         GLib.timeout_add(60 * 60 * 1000, klass.__do_check)
 
     def Menu(self, songs, songlist, library):
-        menu = SongsMenu(
-            library, songs, accels=songlist.accelerators, parent=self)
+        menu = SongsMenu(library, songs, parent=self)
         if len(songs) == 1:
             item = qltk.MenuItem(_("_Download..."), Gtk.STOCK_CONNECT)
             item.connect('activate', self.__download, songs[0]("~uri"))
@@ -361,7 +361,7 @@ class AudioFeeds(Browser, Gtk.VBox):
         swin.add(view)
         self.pack_start(swin, True, True, 0)
 
-        new = Gtk.Button(stock=Gtk.STOCK_NEW)
+        new = Button(_("_New"), Gtk.STOCK_ADD, Gtk.IconSize.MENU)
         new.connect('clicked', self.__new_feed)
         view.get_selection().connect('changed', self.__changed)
         view.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
@@ -381,7 +381,9 @@ class AudioFeeds(Browser, Gtk.VBox):
         self.connect_object('destroy', self.__save, view)
 
         self.pack_start(Alignment(new, left=3, bottom=3), False, True, 0)
-        self.show_all()
+
+        for child in self.get_children():
+            child.show_all()
 
     def __drag_motion(self, view, ctx, x, y, time):
         targets = [t.name() for t in ctx.list_targets()]
