@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2010, 2012 Christoph Reiter
+# Copyright 2010, 2012, 2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -17,6 +17,8 @@ from quodlibet import util
 from quodlibet.browsers.albums import AlbumTagCompletion
 from quodlibet.browsers._base import Browser
 from quodlibet.parse import XMLFromPattern, Query
+from quodlibet.qltk.models import ObjectTreeStore, ObjectModelFilter
+from quodlibet.qltk.models import ObjectModelSort
 from quodlibet.qltk.searchbar import SearchBarBox
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.tagscombobox import TagsComboBoxEntry
@@ -24,12 +26,13 @@ from quodlibet.qltk.views import AllTreeView, BaseView
 from quodlibet.qltk.x import ScrolledWindow, Alignment, SymbolicIconImage
 from quodlibet.util.collection import Album
 from quodlibet.util.library import background_filter
+from quodlibet.util.path import mkdir
 from quodlibet.util.thumbnails import scale
 
 
 EMPTY = _("Songs not in an album")
 ALBUM_PATTERN = r"""
-\<b\><album|\<i\><album>\</i\>|%s>\</b\><date| \<small\>(<date>)\</small\>>
+\<b\><album|<album>|%s>\</b\><date| \<small\>(<date>)\</small\>>
 \<small\><~discs|<~discs> - ><~tracks> - <~long-length>\</small\>""" % EMPTY
 ALBUM_PATTERN = ALBUM_PATTERN.lstrip()
 
@@ -39,7 +42,7 @@ COUNT_PATTERN = " <span size='small' color='#777'>(%s)</span>"
 
 PAT = XMLFromPattern(ALBUM_PATTERN)
 ALBUM_QUERIES = os.path.join(const.USERDIR, "lists", "album_queries")
-util.mkdir(os.path.dirname(ALBUM_QUERIES))
+mkdir(os.path.dirname(ALBUM_QUERIES))
 
 
 def get_headers():
@@ -271,10 +274,8 @@ def build_tree(tags, albums, cache=None):
     return tree
 
 
-class StoreUtils(object):
-    """TreeModelFilter can't be subclassed, otherwise this would be a mixin"""
+class CollectionModelMixin(object):
 
-    @staticmethod
     def get_path_for_album(self, album):
         """Returns the path for an album or None"""
 
@@ -290,11 +291,9 @@ class StoreUtils(object):
         self.foreach(func, res)
         return res[0]
 
-    @staticmethod
     def get_albums_for_path(self, path):
-        return StoreUtils.get_albums_for_iter(self, self[path].iter)
+        return self.get_albums_for_iter(self.get_iter(path))
 
-    @staticmethod
     def get_albums_for_iter(self, iter_):
         row = self[iter_]
 
@@ -307,13 +306,11 @@ class StoreUtils(object):
             if isinstance(obj, Album):
                 albums.add(obj)
             else:
-                albums.update(
-                    StoreUtils.get_albums_for_iter(self, child.iter))
+                albums.update(self.get_albums_for_iter(child.iter))
         return albums
 
-    @staticmethod
     def get_markup(self, tags, iter_):
-        obj = self[iter_][0]
+        obj = self.get_value(iter_, 0)
         if isinstance(obj, Album):
             return PAT % obj
 
@@ -326,17 +323,24 @@ class StoreUtils(object):
             else:
                 markup = MULTI_PATTERN % util.escape(tag)
 
-        num = len(StoreUtils.get_albums_for_iter(self, iter_))
+        num = len(self.get_albums_for_iter(iter_))
         return markup + COUNT_PATTERN % num
 
-    @staticmethod
     def get_album(self, iter_):
-        obj = self[iter_][0]
+        obj = self.get_value(iter_, 0)
         if isinstance(obj, Album):
             return obj
 
 
-class CollectionTreeStore(Gtk.TreeStore):
+class CollectionFilterModel(ObjectModelFilter, CollectionModelMixin):
+    pass
+
+
+class CollectionSortModel(ObjectModelSort, CollectionModelMixin):
+    pass
+
+
+class CollectionTreeStore(ObjectTreeStore, CollectionModelMixin):
     def __init__(self):
         super(CollectionTreeStore, self).__init__(object)
         self.__tags = []
@@ -461,7 +465,8 @@ class CollectionView(AllTreeView):
             view.expand_row(children[0].path, False)
 
     def select_album(self, album, unselect=True):
-        path = StoreUtils.get_path_for_album(self.get_model(), album)
+        model = self.get_model()
+        path = model.get_path_for_album(album)
         if path is not None:
             self.select_path(path, unselect)
 
@@ -473,6 +478,7 @@ class CollectionView(AllTreeView):
             self.expand_row(Gtk.TreePath(tuple(path_idx[:i + 1])), False)
         self.scroll_to_cell(path, use_align=True, row_align=0.5)
         selection = self.get_selection()
+        assert selection
         if unselect:
             selection.unselect_all()
             self.set_cursor(path)
@@ -481,11 +487,11 @@ class CollectionView(AllTreeView):
 
     def get_selected_albums(self):
         selection = self.get_selection()
+        assert selection
         model, paths = selection.get_selected_rows()
-        model = self.get_model()
         albums = set()
         for path in paths:
-            albums.update(StoreUtils.get_albums_for_path(model, path))
+            albums.update(model.get_albums_for_path(path))
         return albums
 
 
@@ -561,8 +567,8 @@ class CollectionBrowser(Browser, Gtk.VBox, util.InstanceTracker):
         sw.set_shadow_type(Gtk.ShadowType.IN)
         self.view = view = CollectionView()
         view.set_headers_visible(False)
-        model_sort = Gtk.TreeModelSort(model=self.__model)
-        model_filter = model_sort.filter_new()
+        model_sort = CollectionSortModel(model=self.__model)
+        model_filter = CollectionFilterModel(child_model=model_sort)
         self.__filter = None
         self.__bg_filter = background_filter()
         model_filter.set_visible_func(self.__parse_query)
@@ -594,18 +600,30 @@ class CollectionBrowser(Browser, Gtk.VBox, util.InstanceTracker):
         column = Gtk.TreeViewColumn("albums")
 
         def cell_data(column, cell, model, iter_, data):
-            markup = StoreUtils.get_markup(model, self.__model.tags, iter_)
+            markup = model.get_markup(self.__model.tags, iter_)
             cell.markup = markup
             cell.set_property('markup', markup)
 
+        def get_scaled_cover(album):
+            # XXX: Cache this somewhere else
+            cover = None
+            if not hasattr(album, "_scaled_cover"):
+                album.scan_cover()
+                if album.cover:
+                    cover = scale(album.cover, (25, 25))
+                    album._scaled_cover = cover
+            else:
+                cover = album._scaled_cover
+            return cover
+
         def cell_data_pb(column, cell, model, iter_, data):
-            album = StoreUtils.get_album(model, iter_)
+            album = model.get_album(iter_)
             if album is None:
                 cell.set_property('stock_id', Gtk.STOCK_DIRECTORY)
             else:
-                album.scan_cover()
-                if album.cover:
-                    cell.set_property('pixbuf', scale(album.cover, (25, 25)))
+                cover = get_scaled_cover(album)
+                if cover:
+                    cell.set_property('pixbuf', cover)
                 else:
                     cell.set_property('stock_id', Gtk.STOCK_CDROM)
 
@@ -636,7 +654,7 @@ class CollectionBrowser(Browser, Gtk.VBox, util.InstanceTracker):
         hbox.pack_start(prefs, False, True, 0)
 
         if main:
-            self.pack_start(Alignment(hbox, left=3, top=3), False, True, 0)
+            self.pack_start(Alignment(hbox, left=6, top=6), False, True, 0)
         else:
             self.pack_start(hbox, False, True, 0)
 
@@ -676,7 +694,7 @@ class CollectionBrowser(Browser, Gtk.VBox, util.InstanceTracker):
         if isinstance(obj, Album):
             return check_album(obj)
         else:
-            for album in StoreUtils.get_albums_for_iter(model, iter_):
+            for album in model.get_albums_for_iter(iter_):
                 if check_album(album):
                     return True
             return False
@@ -698,9 +716,7 @@ class CollectionBrowser(Browser, Gtk.VBox, util.InstanceTracker):
     def __drag_data_get(self, view, ctx, sel, tid, etime):
         songs = self.__get_selected_songs()
         if tid == 1:
-            filenames = [song("~filename") for song in songs]
-            type_ = Gdk.atom_intern("text/x-quodlibet-songs", True)
-            sel.set(type_, 8, "\x00".join(filenames))
+            qltk.selection_set_songs(sel, songs)
         else:
             sel.set_uris([song("~uri") for song in songs])
 
@@ -732,11 +748,9 @@ class CollectionBrowser(Browser, Gtk.VBox, util.InstanceTracker):
         return songs
 
     def __selection_changed(self, selection):
-        def idle_emit():
-            songs = self.__get_selected_songs(False)
-            if songs:
-                self.emit('songs-selected', songs, None)
-        GLib.idle_add(idle_emit)
+        songs = self.__get_selected_songs(False)
+        if songs is not None:
+            GLib.idle_add(self.emit, 'songs-selected', songs, None)
 
     def can_filter_albums(self):
         return True
@@ -749,10 +763,7 @@ class CollectionBrowser(Browser, Gtk.VBox, util.InstanceTracker):
             self.view.select_album(album, unselect=False)
 
     def unfilter(self):
-        model = self.view.get_model()
-        root = model.get_iter_root()
-        if root:
-            self.view.set_cursor(model[root].path)
+        pass
 
     def activate(self):
         self.view.get_selection().emit('changed')
