@@ -12,7 +12,7 @@ import urllib2
 import urllib
 import itertools
 
-from gi.repository import Gtk, GLib
+from gi.repository import Gtk, GLib, Pango
 
 from quodlibet import const
 from quodlibet import qltk
@@ -28,6 +28,7 @@ from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.notif import Task
 from quodlibet.util import copool, gobject_weak, sanitize_tags
+from quodlibet.util.string import decode, encode
 from quodlibet.util.uri import URI
 from quodlibet.qltk.views import AllTreeView
 from quodlibet.qltk.searchbar import SearchBarBox
@@ -36,7 +37,8 @@ from quodlibet.qltk.x import MenuItem, Alignment, ScrolledWindow
 from quodlibet.qltk.x import SymbolicIconImage, SeparatorMenuItem
 from quodlibet.qltk.menubutton import MenuButton
 
-STATION_LIST_URL = "http://quodlibet.googlecode.com/files/radiolist.bz2"
+STATION_LIST_URL = \
+    "http://bitbucket.org/lazka/quodlibet/downloads/radiolist.bz2"
 STATIONS_FAV = os.path.join(const.USERDIR, "stations")
 STATIONS_ALL = os.path.join(const.USERDIR, "stations_all")
 
@@ -103,7 +105,7 @@ class IRFile(RemoteFile):
         for tag in ["title", "artist", "~format"]:
             value = self.get(tag)
             if value is not None:
-                lines.append("%s=%s" % (tag, util.encode(value)))
+                lines.append("%s=%s" % (tag, encode(value)))
         return "\n".join(lines)
 
     def can_change(self, k=None):
@@ -302,12 +304,16 @@ def parse_taglist(data):
             station = IRFile(value)
             continue
 
-        value = util.decode(value)
+        value = decode(value)
         san = sanitize_tags({key: value}, stream=True).items()
         if not san:
             continue
 
         key, value = san[0]
+        if key == "~listenerpeak":
+            key = "~#listenerpeak"
+            value = int(value)
+
         if isinstance(value, str):
             value = value.decode("utf-8")
             if value not in station.list(key):
@@ -321,12 +327,6 @@ def parse_taglist(data):
     return stations
 
 
-def sort_stations(station):
-    bitrate = station("~#bitrate", 96)
-    listeners = int(station("~listenerpeak", 20))
-    return (listeners >= 20, bitrate, listeners)
-
-
 class AddNewStation(GetStringDialog):
     def __init__(self, parent):
         super(AddNewStation, self).__init__(
@@ -412,6 +412,68 @@ class GenreFilter(object):
         return self.GENRES[key][0]
 
 
+class CloseButton(Gtk.Button):
+    """Reimplementation of 3.10 close button for InfoBar."""
+
+    def __init__(self):
+        image = Gtk.Image(visible=True, can_focus=False,
+                          icon_name="window-close-symbolic")
+
+        # Gtk.Align.CENTER (3) broken on Windows with gtk 3.8
+        super(CloseButton, self).__init__(
+            visible=False, can_focus=True, image=image,
+            relief=Gtk.ReliefStyle.NONE, valign=3)
+
+        ctx = self.get_style_context()
+        ctx.add_class("raised")
+        ctx.add_class("close")
+
+
+class QuestionBar(Gtk.InfoBar):
+    """A widget which suggest to download the radio list if
+    no radio stations are present.
+
+    Connect to Gtk.InfoBar::response and check for RESPONSE_LOAD
+    as response id.
+    """
+
+    RESPONSE_LOAD = 1
+
+    def __init__(self):
+        super(QuestionBar, self).__init__()
+        self.connect("response", self.__response)
+        self.set_message_type(Gtk.MessageType.QUESTION)
+
+        label = Gtk.Label(label=_(
+            "Would you like to load a list of popular radio stations?"))
+        label.set_line_wrap(True)
+        label.show()
+        content = self.get_content_area()
+        content.add(label)
+
+        self.add_button(_("_Load Stations"), self.RESPONSE_LOAD)
+
+        try:
+            self.set_show_close_button(True)
+        except (AttributeError, GLib.GError):
+            # On Windows we use GTK+3.8 but the typelib is from 3.10.
+            # As a result libgirepository throws GError here because the
+            # library symbol is missing
+
+            # < gtk3.10
+            action = self.get_action_area()
+            close_button = CloseButton()
+            close_button.show()
+            self.add_action_widget(close_button, Gtk.ResponseType.CLOSE)
+            # FIXME: doesn't seem to have any effect
+            action.set_child_packing(
+                close_button, False, True, 2, Gtk.PackType.START)
+
+    def __response(self, bar, response_id):
+        if response_id == Gtk.ResponseType.CLOSE:
+            bar.hide()
+
+
 class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
     __gsignals__ = Browser.__gsignals__
 
@@ -425,7 +487,7 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
     accelerated_name = _("_Internet Radio")
     priority = 16
     headers = "title artist ~people grouping genre website ~format " \
-        "channel-mode ~listenerpeak".split()
+        "channel-mode".split()
 
     TYPE, STOCK, KEY, NAME = range(4)
     TYPE_FILTER, TYPE_ALL, TYPE_FAV, TYPE_SEP, TYPE_NOCAT = range(5)
@@ -545,6 +607,7 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
         column.add_attribute(renderpb, "stock_id", self.STOCK)
 
         render = Gtk.CellRendererText()
+        render.set_property('ellipsize', Pango.EllipsizeMode.END)
         view.append_column(column)
         column.pack_start(render, True)
         column.add_attribute(render, "text", self.NAME)
@@ -561,20 +624,39 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
         box.pack_start(search, True, True, 0)
         box.pack_start(button, False, True, 0)
         if main:
-            self.pack_start(Alignment(box, left=3, right=3, top=3),
+            self.pack_start(Alignment(box, left=6, right=6, top=6),
                             True, True, 0)
         else:
             self.pack_start(box, True, True, 0)
         self.__filter_list = scrolled_window
 
+        def qbar_response(infobar, response_id):
+            if response_id == infobar.RESPONSE_LOAD:
+                self.__update()
+
+        self.qbar = QuestionBar()
+        self.qbar.connect("response", qbar_response)
+        if self._is_library_empty():
+            self.qbar.show()
+
         self.show_all()
 
+    def _is_library_empty(self):
+        return not len(self.__stations) and not len(self.__fav_stations)
+
     def pack(self, songpane):
         container = Gtk.VBox(spacing=6)
+
         pane = qltk.RHPaned()
         pane.pack1(self.__filter_list, resize=False, shrink=False)
         pane.show_all()
-        pane.pack2(songpane, resize=True, shrink=False)
+
+        songbox = Gtk.VBox(spacing=6)
+        songbox.pack_start(songpane, True, True, 0)
+        songbox.pack_start(self.qbar, False, True, 0)
+        songbox.show()
+
+        pane.pack2(songbox, resize=True, shrink=False)
         container.pack_start(self, False, True, 0)
         container.pack_start(pane, True, True, 0)
         return container
@@ -582,9 +664,11 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
     def unpack(self, container, songpane):
         container.remove(self)
         pane = container.get_children()[0]
-        pane.remove(songpane)
+        box = pane.get_children()[1]
+        box.remove(songpane)
 
     def __update(self, *args):
+        self.qbar.hide()
         copool.add(download_taglist, self.__update_done,
                    cofuncid="radio-load", funcid="radio-load")
 
@@ -593,16 +677,42 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
             print_w("Loading remote station list failed.")
             return
 
-        # take the best 4000
-        stations.sort(key=sort_stations, reverse=True)
-        stations = stations[:4000]
+        # filter stations based on quality, listenercount
+        def filter_stations(station):
+            peak = station.get("~#listenerpeak", 0)
+            if peak < 10:
+                return False
+            aac = "AAC" in station("~format")
+            bitrate = station("~#bitrate", 50)
+            if (aac and bitrate < 40) or (not aac and bitrate < 60):
+                return False
+            return True
+        stations = filter(filter_stations, stations)
 
-        # remove the tags only used for ranking
+        # group them based on the title
+        groups = {}
         for s in stations:
-            s.pop("~listenerpeak", None)
+            key = s("~title~artist")
+            groups.setdefault(key, []).append(s)
 
+        # keep at most 2 URLs for each group
+        stations = []
+        for key, sub in groups.iteritems():
+            sub.sort(key=lambda s: s.get("~#listenerpeak", 0), reverse=True)
+            stations.extend(sub[:2])
+
+        # only keep the ones in at least one category
+        all_ = [self.filters.query(k) for k in self.filters.keys()]
+        assert all_
+        anycat_filter = reduce(lambda x, y: x | y, all_)
+        stations = filter(anycat_filter.search, stations)
+
+        # remove listenerpeak
+        for s in stations:
+            s.pop("~#listenerpeak", None)
+
+        # update the libraries
         stations = dict(((s.key, s) for s in stations))
-
         # don't add ones that are in the fav list
         for fav in self.__fav_stations.iterkeys():
             stations.pop(fav, None)
@@ -720,8 +830,7 @@ class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
 
     def Menu(self, songs, songlist, library):
         menu = SongsMenu(self.__librarian, songs, playlists=False, remove=True,
-                         queue=False, accels=songlist.accelerators,
-                         devices=False, parent=self)
+                         queue=False, devices=False, parent=self)
 
         menu.prepend(SeparatorMenuItem())
 
