@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
-#                2012 Nick Boultbee
+#           2012-2013 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -15,15 +15,21 @@ import shutil
 import time
 import re
 
+from ._image import ImageContainer
+
 from quodlibet import const
 from quodlibet import util
 from quodlibet import config
+from quodlibet.util.path import mkdir, fsdecode, fsencode, mtime, expanduser
+from quodlibet.util.path import normalize_path
+from quodlibet.util.string import encode
 
 from quodlibet.util.uri import URI
-from quodlibet.util import HashableDict
 from quodlibet.util import human_sort_key as human
 from quodlibet.util.dprint import print_d, print_w
 
+from quodlibet.util.cover.manager import cover_plugins
+
 # Used by __init__.py
 from quodlibet.util.tags import STANDARD_TAGS as USEFUL_TAGS
 from quodlibet.util.tags import MACHINE_TAGS
@@ -52,11 +58,8 @@ PEOPLE_SORT = [TAG_TO_SORT.get(k, k) for k in PEOPLE]
 
 FILESYSTEM_TAGS = "~filename ~basename ~dirname".split()
 
-# tags that should alone identify an album, ordered by descending preference
-UNIQUE_ALBUM_IDENTIFIERS = ["musicbrainz_albumid", "labelid"]
 
-
-class AudioFile(dict):
+class AudioFile(dict, ImageContainer):
     """An audio file. It looks like a dict, but implements synthetic
     and tied tags via __call__ rather than __getitem__. This means
     __getitem__, get, and so on can be used for efficiency.
@@ -91,6 +94,7 @@ class AudioFile(dict):
     @util.cached_property
     def album_key(self):
         return (human(self("albumsort", "")),
+                human(self("albumartistsort", "")),
                 self.get("album_grouping_key") or self.get("labelid") or
                 self.get("musicbrainz_albumid") or "")
 
@@ -110,7 +114,7 @@ class AudioFile(dict):
         elif tag == "artistsort":
             return artist_sort
         elif tag in FILESYSTEM_TAGS:
-            return lambda song: util.fsdecode(song(tag), note=False)
+            return lambda song: fsdecode(song(tag), note=False)
         elif tag.startswith("~#") and "~" not in tag[2:]:
             return lambda song: song(tag)
         return lambda song: human(song(tag))
@@ -149,36 +153,6 @@ class AudioFile(dict):
     def mountpoint(self):
         return self["~mountpoint"]
 
-    def _album_id_values(self, use_artist=False):
-        """Returns a "best attempt" conjunction (=AND) of album identifiers
-
-        Tries the (probably) most specific keys first, then gets less accurate
-        but more verbose (eg artist="foo" AND album="bar").
-
-        if use_artist is True, the track artist will also be used as a key if
-        necessary. This breaks compilations, but does well for overloaded
-        album names (eg "Greatest Hits")
-        """
-        ret = HashableDict()
-        for key in UNIQUE_ALBUM_IDENTIFIERS:
-            val = self.get(key)
-            if val:
-                ret[key] = val
-                break
-        if not ret and "album" in self:
-            # Add helpful identifiers where they exist
-            for key in ["album", "albumartist", "album_grouping_key"]:
-                val = self.get(key)
-                if val:
-                    ret[key] = val
-            if use_artist and "albumartist" not in ret and "artist" in self:
-                ret["artist"] = self.get("artist")
-        return ret
-
-    """Returns a dict of tag:value items that together identify this song's
-    album"""
-    album_id_values = property(_album_id_values)
-
     def __cmp__(self, other):
         if not other:
             return -1
@@ -259,7 +233,7 @@ class AudioFile(dict):
                 else:
                     return util.format_time(length)
             elif key == "#rating":
-                return dict.get(self, "~" + key, const.DEFAULT_RATING)
+                return dict.get(self, "~" + key, config.RATINGS.default)
             elif key == "rating":
                 return util.format_rating(self("~#rating"))
             elif key == "people":
@@ -414,8 +388,8 @@ class AudioFile(dict):
         filename = self.comma("title").replace('/', '')[:128] + '.lyric'
         sub_dir = ((self.comma("lyricist") or self.comma("artist"))
                   .replace('/', '')[:128])
-        path = os.path.join(util.expanduser("~/.lyrics"), sub_dir, filename)
-        return util.fsencode(path)
+        path = os.path.join(expanduser("~/.lyrics"), sub_dir, filename)
+        return fsencode(path)
 
     def comma(self, key):
         """Get all values of a tag, separated by commas. Synthetic
@@ -466,6 +440,25 @@ class AudioFile(dict):
         else:
             return self.list(key)
 
+    def as_lowercased(self):
+        """Returns a new AudioFile with all keys lowercased / values merged.
+
+        Useful for tag writing for case insensitive tagging formats like
+        APEv2 or VorbisComment.
+        """
+
+        merged = AudioFile()
+        text = {}
+        for key, value in self.iteritems():
+            lower = key.lower()
+            if key.startswith("~#"):
+                merged[lower] = value
+            else:
+                text.setdefault(lower, []).extend(value.split("\n"))
+        for key, values in text.items():
+            merged[key] = "\n".join(values)
+        return merged
+
     def exists(self):
         """Return true if the file still exists (or we can't tell)."""
 
@@ -474,8 +467,8 @@ class AudioFile(dict):
     def valid(self):
         """Return true if the file cache is up-to-date (checked via
         mtime), or we can't tell."""
-        return (self.get("~#mtime", 0) and
-                self["~#mtime"] == util.mtime(self["~filename"]))
+        return (bool(self.get("~#mtime", 0)) and
+                self["~#mtime"] == mtime(self["~filename"]))
 
     def mounted(self):
         """Return true if the disk the file is on is mounted, or
@@ -513,13 +506,15 @@ class AudioFile(dict):
         directly; use library.rename instead."""
 
         if os.path.isabs(newname):
-            util.mkdir(os.path.dirname(newname))
+            mkdir(os.path.dirname(newname))
         else:
             newname = os.path.join(self('~dirname'), newname)
+
         if not os.path.exists(newname):
             shutil.move(self['~filename'], newname)
-        elif os.path.realpath(newname) != self['~filename']:
+        elif normalize_path(os.path.realpath(newname)) != self['~filename']:
             raise ValueError
+
         self.sanitize(newname)
 
     def website(self):
@@ -543,8 +538,8 @@ class AudioFile(dict):
             else:
                 artist = util.escape("+".join(self("artist").split()))
                 album = util.escape("+".join(self("album").split()))
-                artist = util.encode(artist)
-                album = util.encode(album)
+                artist = encode(artist)
+                album = encode(album)
                 artist = "%22" + ''.join(map(esc, artist)) + "%22"
                 album = "%22" + ''.join(map(esc, album)) + "%22"
                 text += artist + "+" + album
@@ -569,6 +564,7 @@ class AudioFile(dict):
             raise ValueError("Unknown filename!")
         if self.is_file:
             self["~filename"] = os.path.realpath(self["~filename"])
+            self["~filename"] = normalize_path(self["~filename"])
             # Find mount point (terminating at "/" if necessary)
             head = self["~filename"]
             while "~mountpoint" not in self:
@@ -616,7 +612,7 @@ class AudioFile(dict):
                     if isinstance(v2, str):
                         s.append("%s=%s" % (k, v2))
                     else:
-                        s.append("%s=%s" % (k, util.encode(v2)))
+                        s.append("%s=%s" % (k, encode(v2)))
         for k in (INTERN_NUM_DEFAULT - set(self.keys())):
             s.append("%s=%d" % (k, self.get(k, 0)))
         if "~#rating" not in self:
@@ -683,107 +679,10 @@ class AudioFile(dict):
                 if key in self:
                     del(self[key])
 
-    # These should remain outside the loop below for performance reasons
-    __cover_subdirs = frozenset(
-        ["scan", "scans", "images", "covers", "artwork"])
-    __cover_exts = frozenset(["jpg", "jpeg", "png", "gif"])
-
-    __cover_positive_words = ["front", "cover", "frontcover", "jacket",
-            "folder", "albumart", "edited"]
-    __cover_positive_regexes = frozenset(
-            map(lambda s: re.compile(r'(\b|_)' + s + r'(\b|_)'),
-            __cover_positive_words))
-    __cover_negative_regexes = frozenset(
-            map(lambda s: re.compile(r'(\b|_|)' + s + r'(\b|_)'),
-            ["back", "inlay", "inset", "inside"]))
-
     def find_cover(self):
         """Return a file-like containing cover image data, or None if
         no cover is available."""
-        if not self.is_file:
-            return
-
-        # If preferred, check for picture stored in the metadata...
-        if (config.getboolean("albumart", "prefer_embedded") and
-                "~picture" in self):
-            print_d("Preferring embedded art for %s" % self("~filename"))
-            return self.get_format_cover()
-
-        base = self('~dirname')
-        images = []
-
-        # Issue 374: Specify artwork filename
-        if config.getboolean("albumart", "force_filename"):
-            path = os.path.join(base, config.get("albumart", "filename"))
-            if os.path.isfile(path):
-                images = [(100, path)]
-        else:
-            get_ext = lambda s: os.path.splitext(s)[1].lstrip('.')
-
-            entries = []
-            try:
-                entries = os.listdir(base)
-            except EnvironmentError:
-                pass
-
-            fns = []
-            for entry in entries:
-                lentry = entry.lower()
-                if get_ext(lentry) in self.__cover_exts:
-                    fns.append((None, entry))
-                if lentry in self.__cover_subdirs:
-                    subdir = os.path.join(base, entry)
-                    sub_entries = []
-                    try:
-                        sub_entries = os.listdir(subdir)
-                    except EnvironmentError:
-                        pass
-                    for sub_entry in sub_entries:
-                        lsub_entry = sub_entry.lower()
-                        if get_ext(lsub_entry) in self.__cover_exts:
-                            fns.append((entry, sub_entry))
-
-            for sub, fn in fns:
-                score = 0
-                lfn = fn.lower()
-                # check for the album label number
-                if "labelid" in self and self["labelid"].lower() in lfn:
-                    score += 20
-
-                # Track-related keywords
-                keywords = [k.lower().strip() for k in [self("artist"),
-                            self("albumartist"), self("album")] if len(k) > 1]
-                score += 2 * sum(map(lfn.__contains__, keywords))
-
-                # Generic keywords
-                score += 3 * sum(r.search(lfn) is not None
-                                 for r in self.__cover_positive_regexes)
-
-                negs = sum(r.search(lfn) is not None
-                           for r in self.__cover_negative_regexes)
-                score -= 2 * negs
-                #print("[%s - %s]: Album art \"%s\" scores %d (%s neg)." % (
-                #        self("artist"), self("title"), fn, score, negs))
-                if score > 0:
-                    if sub is not None:
-                        fn = os.path.join(sub, fn)
-                    images.append((score, os.path.join(base, fn)))
-            images.sort(reverse=True)
-
-        for score, path in images:
-            # could be a directory
-            if not os.path.isfile(path):
-                continue
-            try:
-                return file(path, "rb")
-            except IOError:
-                print_w(_("Failed reading album art \"%s\"") % path)
-
-        if "~picture" in self:
-            # Otherwise, we might have a picture stored in the metadata...
-            return self.get_format_cover()
-
-        return None
+        return cover_plugins.acquire_cover_sync(self)
 
     def replay_gain(self, profiles, pre_amp_gain=0, fallback_gain=0):
         """Return the computed Replay Gain scale factor.
@@ -839,10 +738,10 @@ class AudioFile(dict):
 
     def __set_bookmarks(self, marks):
         result = []
-        for time, mark in marks:
-            if time < 0:
+        for time_, mark in marks:
+            if time_ < 0:
                 raise ValueError("mark times must be positive")
-            result.append(u"%s %s" % (util.format_time(time), mark))
+            result.append(u"%s %s" % (util.format_time(time_), mark))
         result = u"\n".join(result)
         if result:
             self["~bookmark"] = result
@@ -857,3 +756,12 @@ class AudioFile(dict):
            marks = song.bookmarks
            marks.append(...)
            song.bookmarks = marks""")
+
+
+# Looks like the real thing.
+DUMMY_SONG = AudioFile({
+    '~#length': 234, '~filename': '/dev/null',
+    'artist': 'The Artist', 'album': 'An Example Album',
+    'title': 'First Track', 'tracknumber': 1,
+    'date': '2010-12-31',
+})
