@@ -1,5 +1,5 @@
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
-#           2013 Christoph Reiter
+#           2009-2014 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,12 +9,14 @@ import sys
 import base64
 
 import mutagen
+from mutagen.flac import Picture, error as FLACError
 
 from quodlibet import config
 from quodlibet import const
-from quodlibet import util
-
+from quodlibet.config import RATINGS
+from quodlibet.util.path import get_temp_cover_file
 from quodlibet.formats._audio import AudioFile
+from quodlibet.formats._image import EmbeddedImage, APICType
 
 
 # Migrate old layout
@@ -26,6 +28,8 @@ class MutagenVCFile(AudioFile):
     format = "Unknown Mutagen + vorbiscomment"
     MutagenType = None
 
+    can_change_images = True
+
     def __init__(self, filename, audio=None):
         # If we're done a type probe, use the results of that to avoid
         # reopening the file.
@@ -36,6 +40,7 @@ class MutagenVCFile(AudioFile):
             self["~#bitrate"] = int(audio.info.bitrate / 1000)
         except AttributeError:
             pass
+        # mutagen keys are lower cased
         for key, value in (audio.tags or {}).items():
             self[key] = "\n".join(value)
         self.__post_read()
@@ -87,11 +92,11 @@ class MutagenVCFile(AudioFile):
                     del(self[key])
 
         if "metadata_block_picture" in self:
-            self["~picture"] = "y"
+            self.has_images = True
             del(self["metadata_block_picture"])
 
         if "coverart" in self:
-            self["~picture"] = "y"
+            self.has_images = True
             del(self["coverart"])
 
         if "coverartmime" in self:
@@ -100,11 +105,44 @@ class MutagenVCFile(AudioFile):
         self.__post_read_total("tracktotal", "totaltracks", "tracknumber")
         self.__post_read_total("disctotal", "totaldiscs", "discnumber")
 
-    def get_format_cover(self):
+    def get_images(self):
         try:
-            from mutagen.flac import Picture
-        except ImportError:
-            return
+            audio = self.MutagenType(self["~filename"])
+        except EnvironmentError:
+            return []
+
+        # metadata_block_picture
+        images = []
+        for data in audio.get("metadata_block_picture", []):
+            try:
+                cover = Picture(base64.b64decode(data))
+            except (TypeError, FLACError):
+                continue
+
+            f = get_temp_cover_file(cover.data)
+            images.append(EmbeddedImage(
+                f, cover.mime, cover.width, cover.height, cover.depth,
+                cover.type))
+
+        # coverart + coverartmime
+        cover = audio.get("coverart")
+        try:
+            cover = cover and base64.b64decode(cover[0])
+        except TypeError:
+            cover = None
+
+        if cover:
+            mime = audio.get("coverartmime")
+            mime = (mime and mime[0]) or "image/"
+            f = get_temp_cover_file(cover)
+            images.append(EmbeddedImage(f, mime))
+
+        images.sort(key=lambda c: c.sort_key)
+
+        return images
+
+    def get_primary_image(self):
+        """Returns the primary embedded image"""
 
         try:
             audio = self.MutagenType(self["~filename"])
@@ -112,43 +150,93 @@ class MutagenVCFile(AudioFile):
             return None
 
         pictures = []
-        for data in audio.tags.get("metadata_block_picture", []):
+        for data in audio.get("metadata_block_picture", []):
             try:
                 pictures.append(Picture(base64.b64decode(data)))
-            except TypeError:
+            except (TypeError, FLACError):
                 pass
 
         cover = None
         for pic in pictures:
-            if pic.type == 3:
-                cover = pic.data
+            if pic.type == APICType.COVER_FRONT:
+                cover = pic
                 break
-            cover = cover or pic.data
+            cover = cover or pic
+
+        if cover:
+            f = get_temp_cover_file(cover.data)
+            return EmbeddedImage(
+                f, cover.mime, cover.width, cover.height, cover.depth,
+                cover.type)
+
+        cover = audio.get("coverart")
+        try:
+            cover = cover and base64.b64decode(cover[0])
+        except TypeError:
+            cover = None
 
         if not cover:
-            cover = audio.tags.get("coverart")
-            try:
-                cover = cover and base64.b64decode(cover[0])
-            except TypeError:
-                cover = None
-
-        if not cover:
-            if "~picture" in self:
-                del self["~picture"]
+            self.has_images = False
             return
 
-        return util.get_temp_cover_file(cover)
+        mime = audio.get("coverartmime")
+        mime = (mime and mime[0]) or "image/"
+        f = get_temp_cover_file(cover)
+        return EmbeddedImage(f, mime)
+
+    def clear_images(self):
+        """Delete all embedded images"""
+
+        if not self.has_images:
+            return
+
+        try:
+            audio = self.MutagenType(self["~filename"])
+        except EnvironmentError:
+            return
+
+        audio.pop("metadata_block_picture", None)
+        audio.pop("coverart", None)
+        audio.pop("coverartmime", None)
+        audio.save()
+
+        self.has_images = False
+
+    def set_image(self, image):
+        """Replaces all embedded images by the passed image"""
+
+        try:
+            audio = self.MutagenType(self["~filename"])
+            data = image.file.read()
+        except EnvironmentError:
+            return
+
+        pic = Picture()
+        pic.data = data
+        pic.type = APICType.COVER_FRONT
+        pic.mime = image.mime_type
+        pic.width = image.width
+        pic.height = image.height
+        pic.depth = image.color_depth
+
+        audio.pop("coverart", None)
+        audio.pop("coverartmime", None)
+        audio["metadata_block_picture"] = base64.b64encode(pic.write())
+        audio.save()
+
+        self.has_images = True
 
     def can_change(self, k=None):
         if k is None:
             return super(MutagenVCFile, self).can_change(None)
         else:
+            l = k.lower()
             return (super(MutagenVCFile, self).can_change(k) and
-                    k not in ["rating", "playcount",
+                    l not in ["rating", "playcount",
                               "metadata_block_picture",
                               "coverart", "coverartmime"] and
-                    not k.startswith("rating:") and
-                    not k.startswith("playcount:"))
+                    not l.startswith("rating:") and
+                    not l.startswith("playcount:"))
 
     def __prep_write(self, comments):
         email = config.get("editing", "save_email").strip()
@@ -163,19 +251,21 @@ class MutagenVCFile(AudioFile):
         if config.getboolean("editing", "save_to_songs"):
             email = email or const.EMAIL
             rating = self("~#rating")
-            if rating != const.DEFAULT_RATING:
+            if rating != RATINGS.default:
                 comments["rating:" + email] = str(rating)
             playcount = self.get("~#playcount", 0)
             if playcount != 0:
                 comments["playcount:" + email] = str(playcount)
 
     def __prep_write_total(self, comments, main, fallback, single):
+        lower = self.as_lowercased()
+
         for k in [main, fallback, single]:
             if k in comments:
                 del comments[k]
 
-        if single in self:
-            parts = self[single].split("/", 1)
+        if single in lower:
+            parts = lower[single].split("/", 1)
 
             if parts[0]:
                 comments[single] = [parts[0]]
@@ -183,14 +273,14 @@ class MutagenVCFile(AudioFile):
             if len(parts) > 1:
                 comments[main] = [parts[1]]
 
-        if main in self:
-            comments[main] = self.list(main)
+        if main in lower:
+            comments[main] = lower.list(main)
 
-        if fallback in self:
+        if fallback in lower:
             if main in comments:
-                comments[fallback] = self.list(fallback)
+                comments[fallback] = lower.list(fallback)
             else:
-                comments[main] = self.list(fallback)
+                comments[main] = lower.list(fallback)
 
     def write(self):
         audio = self.MutagenType(self["~filename"])
@@ -198,8 +288,10 @@ class MutagenVCFile(AudioFile):
             audio.add_tags()
 
         self.__prep_write(audio.tags)
-        for key in self.realkeys():
-            audio.tags[key] = self.list(key)
+
+        lower = self.as_lowercased()
+        for key in lower.realkeys():
+            audio.tags[key] = lower.list(key)
 
         self.__prep_write_total(audio.tags,
                                 "tracktotal", "totaltracks", "tracknumber")
@@ -302,26 +394,86 @@ class FLACFile(MutagenVCFile):
             audio = FLAC(filename)
         super(FLACFile, self).__init__(filename, audio)
         if audio.pictures:
-            self["~picture"] = "y"
+            self.has_images = True
+
+    def get_images(self):
+        images = super(FLACFile, self).get_images()
+
+        try:
+            tag = FLAC(self["~filename"])
+        except EnvironmentError:
+            return images
+
+        for cover in tag.pictures:
+            fileobj = get_temp_cover_file(cover.data)
+            images.append(EmbeddedImage(
+                fileobj, cover.mime, cover.width, cover.height, cover.depth,
+                cover.type))
+
+        images.sort(key=lambda c: c.sort_key)
+
+        return images
+
+    def get_primary_image(self):
+        """Returns the primary embedded image"""
 
-    def get_format_cover(self):
         try:
             tag = FLAC(self["~filename"])
         except EnvironmentError:
             return None
-        else:
-            covers = tag.pictures
-            if not covers:
-                return super(FLACFile, self).get_format_cover()
 
-            for cover in covers:
-                if cover.type == 3:
-                    pic = cover
-                    break
-            else:
-                pic = covers[0]
+        covers = tag.pictures
+        if not covers:
+            return super(FLACFile, self).get_primary_image()
 
-            return util.get_temp_cover_file(pic.data)
+        covers.sort(key=lambda c: APICType.sort_key(c.type))
+        cover = covers[0]
+
+        fileobj = get_temp_cover_file(cover.data)
+        return EmbeddedImage(
+            fileobj, cover.mime, cover.width, cover.height, cover.depth,
+            cover.type)
+
+    def clear_images(self):
+        """Delete all embedded images"""
+
+        try:
+            tag = FLAC(self["~filename"])
+        except EnvironmentError:
+            return
+
+        tag.clear_pictures()
+        tag.save()
+
+        # clear vcomment tags
+        super(FLACFile, self).clear_images()
+
+        self.has_images = False
+
+    def set_image(self, image):
+        """Replaces all embedded images by the passed image"""
+
+        try:
+            tag = FLAC(self["~filename"])
+            data = image.file.read()
+        except EnvironmentError:
+            return
+
+        pic = Picture()
+        pic.data = data
+        pic.type = APICType.COVER_FRONT
+        pic.mime = image.mime_type
+        pic.width = image.width
+        pic.height = image.height
+        pic.depth = image.color_depth
+
+        tag.add_picture(pic)
+        tag.save()
+
+        # clear vcomment tags
+        super(FLACFile, self).clear_images()
+
+        self.has_images = True
 
     def write(self):
         if ID3 is not None:
