@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2006 Joe Wreschnig, Michael Urman, Iñigo Serna
+#                2014 Nick Boultbee
+#                2014 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -11,12 +13,12 @@ from quodlibet import config
 from quodlibet import util
 from quodlibet import qltk
 
-from quodlibet.plugins import PluginManager
+from quodlibet.plugins import PluginManager, PluginHandler
 from quodlibet.qltk.cbes import ComboBoxEntrySave
 from quodlibet.qltk.ccb import ConfigCheckButton
 
 
-class EditingPluginHandler(GObject.GObject):
+class EditingPluginHandler(GObject.GObject, PluginHandler):
     __gsignals__ = {
         "changed": (GObject.SignalFlags.RUN_LAST, None, ())
     }
@@ -32,14 +34,14 @@ class EditingPluginHandler(GObject.GObject):
         return list(self.__plugins)
 
     def plugin_handle(self, plugin):
-        return issubclass(plugin, self.Kind)
+        return issubclass(plugin.cls, self.Kind)
 
-    def plugin_enable(self, plugin, obj):
-        self.__plugins.append(plugin)
+    def plugin_enable(self, plugin):
+        self.__plugins.append(plugin.cls)
         self.emit("changed")
 
     def plugin_disable(self, plugin):
-        self.__plugins.remove(plugin)
+        self.__plugins.remove(plugin.cls)
         self.emit("changed")
 
 
