@@ -5,7 +5,9 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from gi.repository import Gtk
+import os
+
+from gi.repository import Gtk, GLib, GdkPixbuf
 import mutagen
 
 from quodlibet.qltk import gtk_version, pygobject_version
@@ -14,6 +16,27 @@ from quodlibet import formats
 from quodlibet.util import fver
 
 
+def _set_about_image(dialog, icon_name):
+    # win32 workaround: https://bugzilla.gnome.org/show_bug.cgi?id=721062
+
+    if os.name == "nt":
+        size = 96
+        theme = Gtk.IconTheme.get_default()
+        icon_info = theme.lookup_icon(icon_name, size, 0)
+        if icon_info is None:
+            return
+
+        filename = icon_info.get_filename()
+        try:
+            pb = GdkPixbuf.Pixbuf.new_from_file_at_size(filename, size, size)
+        except GLib.GError:
+            return
+        else:
+            dialog.set_logo(pb)
+    else:
+        dialog.set_logo_icon_name(icon_name)
+
+
 class AboutDialog(Gtk.AboutDialog):
     def __init__(self, parent, player, name, icon):
         super(AboutDialog, self).__init__()
@@ -22,7 +45,7 @@ class AboutDialog(Gtk.AboutDialog):
         self.set_version(const.VERSION)
         self.set_authors(const.AUTHORS)
         self.set_artists(const.ARTISTS)
-        self.set_logo_icon_name(icon)
+        _set_about_image(self, icon)
 
         def chunks(l, n):
             return [l[i:i + n] for i in range(0, len(l), n)]
@@ -43,7 +66,6 @@ class AboutDialog(Gtk.AboutDialog):
         self.set_website(const.WEBSITE)
         self.set_copyright(const.COPYRIGHT + "\n" +
                            "<%s>" % const.SUPPORT_EMAIL)
-        self.get_child().show_all()
 
 
 class AboutQuodLibet(AboutDialog):
