@@ -15,11 +15,8 @@ from quodlibet import util
 from quodlibet.qltk.ccb import ConfigCheckMenuItem
 from quodlibet.qltk.sliderbutton import HSlider
 from quodlibet.qltk.tracker import TimeTracker
-from quodlibet.qltk.x import RadioMenuItem, SeparatorMenuItem
-
-
-SIZE = Gtk.IconSize.LARGE_TOOLBAR
-SUBSIZE = Gtk.IconSize.MENU
+from quodlibet.qltk.x import (RadioMenuItem, SeparatorMenuItem,
+                              SymbolicIconImage)
 
 
 class TimeLabel(Gtk.Label):
@@ -62,9 +59,8 @@ class SeekBar(HSlider):
         hbox = Gtk.HBox(spacing=3)
         l = TimeLabel()
         hbox.pack_start(l, True, True, 0)
-        hbox.pack_start(
-            Gtk.Arrow(Gtk.ArrowType.RIGHT, Gtk.ShadowType.NONE),
-            False, True, 0)
+        arrow = Gtk.Arrow(Gtk.ArrowType.RIGHT, Gtk.ShadowType.NONE)
+        hbox.pack_start(arrow, False, True, 0)
         super(SeekBar, self).__init__(hbox)
 
         self.scale.connect('button-press-event', self.__seek_lock)
@@ -83,14 +79,17 @@ class SeekBar(HSlider):
         m.append(c)
         m.append(SeparatorMenuItem())
         i = qltk.MenuItem(_("_Edit Bookmarks..."), Gtk.STOCK_EDIT)
-        i.connect_object(
-            'activate', bookmarks.EditBookmarks, self, library, player)
+
+        def edit_bookmarks_cb(menu_item):
+            window = bookmarks.EditBookmarks(self, library, player)
+            window.show()
+
+        i.connect('activate', edit_bookmarks_cb)
         m.append(i)
         m.show_all()
-        self.get_child().connect_object(
-            'button-press-event', self.__check_menu, m, player)
-        self.connect_object('popup-menu', self.__popup_menu, m, player,
-                self.get_child().get_child())
+        self.connect_object(
+            'button-press-event', self.__check_menu, m, player, c)
+        self.connect_object('popup-menu', self.__popup_menu, m, player)
 
         timer = TimeTracker(player)
         timer.connect_object('tick', self.__check_time, player)
@@ -98,11 +97,17 @@ class SeekBar(HSlider):
         player.connect('song-started', self.__song_changed, l, m)
         player.connect('seek', self.__seeked)
 
-    def __check_menu(self, menu, event, player):
-        if event.button == 3:
-            return self.__popup_menu(menu, player)
+    def __check_menu(self, menu, event, player, remaining_item):
+        if event.type != Gdk.EventType.BUTTON_PRESS:
+            return
 
-    def __popup_menu(self, menu, player, widget=None):
+        if event.button == Gdk.BUTTON_SECONDARY:
+            return self.__popup_menu(menu, player, event)
+        elif event.button == Gdk.BUTTON_MIDDLE:
+            remaining_item.set_active(not remaining_item.get_active())
+            return True
+
+    def __popup_menu(self, menu, player, event=None):
         for child in menu.get_children()[2:-1]:
             menu.remove(child)
             child.destroy()
@@ -117,12 +122,13 @@ class SeekBar(HSlider):
             items.reverse()
             for i in items:
                 menu.insert(i, 2)
-        time = Gtk.get_current_event_time()
-        if widget:
-            return qltk.popup_menu_under_widget(menu, widget, 3, time)
-        else:
-            menu.popup(None, None, None, None, 3, time)
+
+        if event:
+            menu.popup(None, None, None, None, 3, event.time)
             return True
+        else:
+            time = Gtk.get_current_event_time()
+            return qltk.popup_menu_under_widget(menu, self, 3, time)
 
     def __seeked(self, player, song, ms):
         # If it's not paused, we'll grab it in our next update.
@@ -184,9 +190,8 @@ class SeekBar(HSlider):
 
 class Volume(Gtk.VolumeButton):
     def __init__(self, device):
-        super(Volume, self).__init__()
+        super(Volume, self).__init__(size=Gtk.IconSize.MENU, use_symbolic=True)
 
-        self.props.size = SUBSIZE
         self.set_relief(Gtk.ReliefStyle.NORMAL)
         self.set_adjustment(Gtk.Adjustment(0, 0, 1, 0.05, 0.1, 0))
 
@@ -194,8 +199,6 @@ class Volume(Gtk.VolumeButton):
         device.connect('notify::volume', self.__volume_notify)
         self.set_value(config.getfloat("memory", "volume"))
 
-        self.show_all()
-
         replaygain_menu = ReplayGainMenu(device)
         self.connect('popup-menu', self.__popup, replaygain_menu)
         self.connect_object('button-press-event', self.__volume_button_press,
@@ -207,7 +210,7 @@ class Volume(Gtk.VolumeButton):
         return qltk.popup_menu_under_widget(menu, widget, button, time)
 
     def __volume_button_press(self, menu, event):
-        if event.button == 3:
+        if event.button == Gdk.BUTTON_SECONDARY:
             menu.popup(None, None, None, None, event.button, event.time)
             return True
 
@@ -245,7 +248,7 @@ class ReplayGainMenu(Gtk.Menu):
             item.connect("toggled", self.__changed, player, profile)
             if player.replaygain_profiles[0] == profile:
                 item.set_active(True)
-        self.show_all()
+            item.show()
 
     def __changed(self, item, player, profile):
         if item.get_active():
@@ -259,45 +262,6 @@ class ReplayGainMenu(Gtk.Menu):
         return super(ReplayGainMenu, self).popup(*args)
 
 
-class StopAfterMenu(Gtk.Menu):
-    __menu = None
-
-    def __new__(klass, parent):
-        if klass.__menu is None:
-            return super(StopAfterMenu, klass).__new__(klass)
-        else:
-            return klass.__menu
-
-    def __init__(self, player):
-        if type(self).__menu:
-            return
-        else:
-            type(self).__menu = self
-        super(StopAfterMenu, self).__init__()
-        self.__item = Gtk.CheckMenuItem(_("Stop after this song"))
-        self.__item.set_active(False)
-        self.append(self.__item)
-        player.connect('paused', self.__paused)
-        player.connect_after('song-ended', self.__ended)
-        self.__item.show()
-
-    def __paused(self, player):
-        self.active = False
-
-    def __ended(self, player, song, stopped):
-        if self.active:
-            player.paused = True
-        self.active = False
-
-    def __get_active(self):
-        return self.__item.get_active()
-
-    def __set_active(self, active):
-        return self.__item.set_active(active)
-
-    active = property(__get_active, __set_active)
-
-
 class PlayControls(Gtk.VBox):
     def __init__(self, player, library):
         super(PlayControls, self).__init__(spacing=3)
@@ -307,18 +271,19 @@ class PlayControls(Gtk.VBox):
         upper.set_col_spacings(3)
 
         prev = Gtk.Button()
-        prev.add(Gtk.Image.new_from_stock(Gtk.STOCK_MEDIA_PREVIOUS, SIZE))
+        prev.add(SymbolicIconImage("media-skip-backward",
+                                   Gtk.IconSize.LARGE_TOOLBAR))
         upper.attach(prev, 0, 1, 0, 1)
 
         play = Gtk.ToggleButton()
-        play.add(Gtk.Image.new_from_stock(Gtk.STOCK_MEDIA_PLAY, SIZE))
+        play.add(SymbolicIconImage("media-playback-start",
+                                   Gtk.IconSize.LARGE_TOOLBAR))
         upper.attach(play, 1, 2, 0, 1)
 
-        safter = StopAfterMenu(player)
-
-        next = Gtk.Button()
-        next.add(Gtk.Image.new_from_stock(Gtk.STOCK_MEDIA_NEXT, SIZE))
-        upper.attach(next, 2, 3, 0, 1)
+        next_ = Gtk.Button()
+        next_.add(SymbolicIconImage("media-skip-forward",
+                                    Gtk.IconSize.LARGE_TOOLBAR))
+        upper.attach(next_, 2, 3, 0, 1)
 
         lower = Gtk.Table(rows=1, columns=3, homogeneous=True)
         lower.set_row_spacings(3)
@@ -335,16 +300,12 @@ class PlayControls(Gtk.VBox):
 
         prev.connect_object('clicked', self.__previous, player)
         play.connect('toggled', self.__playpause, player)
-        play.connect('button-press-event', self.__play_button_press, safter)
         play.add_events(Gdk.EventMask.SCROLL_MASK)
         play.connect_object('scroll-event', self.__scroll, player)
-        play.connect_object('popup-menu',
-                            self.__popup, safter, play.get_child())
-        next.connect_object('clicked', self.__next, player)
-        player.connect('song-started', self.__song_started, next, play)
+        next_.connect_object('clicked', self.__next, player)
+        player.connect('song-started', self.__song_started, next_, play)
         player.connect_object('paused', play.set_active, False)
         player.connect_object('unpaused', play.set_active, True)
-        self.show_all()
 
     def __scroll(self, player, event):
         if event.direction in [Gdk.ScrollDirection.UP,
@@ -354,18 +315,6 @@ class PlayControls(Gtk.VBox):
                                  Gdk.ScrollDirection.RIGHT]:
             player.next()
 
-    def __play_button_press(self, activator, event, safter):
-        if event.button == 3:
-            return self.__popup(safter, None, event.button, event.time)
-
-    def __popup(self, safter, widget, button=3, time=None):
-        time = time or Gtk.get_current_event_time()
-        if widget:
-            return qltk.popup_menu_under_widget(safter, widget, button, time)
-        else:
-            safter.popup(None, None, None, None, button, time)
-        return True
-
     def __song_started(self, player, song, next, play):
         play.set_active(not player.paused)
 
