@@ -8,6 +8,7 @@
 import os
 
 from gi.repository import Gtk, GObject, Pango
+from quodlibet.qltk.msg import confirm_action
 
 from quodlibet import config
 from quodlibet import const
@@ -17,9 +18,9 @@ from quodlibet import util
 
 from quodlibet.plugins import PluginManager
 from quodlibet.qltk.ccb import ConfigCheckButton
-from quodlibet.qltk.delete import DeleteDialog
+from quodlibet.qltk.delete import trash_files, TrashMenuItem
 from quodlibet.qltk.edittags import EditTags
-from quodlibet.qltk.filesel import FileSelector
+from quodlibet.qltk.filesel import MainFileSelector, FileSelector
 from quodlibet.qltk.pluginwin import PluginWindow
 from quodlibet.qltk.renamefiles import RenameFiles
 from quodlibet.qltk.tagsfrompath import TagsFromPath
@@ -27,8 +28,9 @@ from quodlibet.qltk.tracknumbers import TrackNumbers
 from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.about import AboutExFalso
 from quodlibet.qltk.songsmenu import SongsMenuPluginHandler
-from quodlibet.qltk.x import Alignment, SeparatorMenuItem
+from quodlibet.qltk.x import Alignment, SeparatorMenuItem, ConfigRHPaned
 from quodlibet.qltk.window import PersistentWindowMixin
+from quodlibet.util.path import mtime, normalize_path
 
 
 class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
@@ -39,7 +41,7 @@ class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
                             None, (object,))
     }
 
-    pm = SongsMenuPluginHandler()
+    pm = SongsMenuPluginHandler(confirm_action)
 
     @classmethod
     def init_plugins(cls):
@@ -48,12 +50,12 @@ class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
     def __init__(self, library, dir=None):
         super(ExFalsoWindow, self).__init__()
         self.set_title("Ex Falso")
-        self.set_default_size(650, 475)
+        self.set_default_size(750, 475)
         self.enable_window_tracking("exfalso")
 
         self.__library = library
 
-        hp = Gtk.HPaned()
+        hp = ConfigRHPaned("memory", "exfalso_paned_position", 1.0)
         hp.set_border_width(0)
         hp.set_position(250)
         hp.show()
@@ -72,11 +74,19 @@ class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
         prefs = Gtk.Button()
         prefs.add(Gtk.Image.new_from_stock(
             Gtk.STOCK_PREFERENCES, Gtk.IconSize.BUTTON))
-        prefs.connect_object('clicked', PreferencesWindow, self)
+
+        def prefs_cb(button):
+            window = PreferencesWindow(self)
+            window.show()
+        prefs.connect('clicked', prefs_cb)
         bbox.pack_start(prefs, False, True, 0)
 
         plugins = qltk.Button(_("_Plugins"), Gtk.STOCK_EXECUTE)
-        plugins.connect_object('clicked', PluginWindow, self)
+
+        def plugin_window_cb(button):
+            window = PluginWindow(self)
+            window.show()
+        plugins.connect('clicked', plugin_window_cb)
         bbox.pack_start(plugins, False, True, 0)
 
         l = Gtk.Label()
@@ -84,7 +94,7 @@ class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
         l.set_ellipsize(Pango.EllipsizeMode.END)
         bbox.pack_start(l, True, True, 0)
 
-        fs = FileSelector(dir)
+        fs = MainFileSelector()
 
         vb.pack_start(fs, True, True, 0)
         vb.pack_start(Alignment(bbox, border=6), False, True, 0)
@@ -95,12 +105,16 @@ class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
         nb = qltk.Notebook()
         nb.show()
         for Page in [EditTags, TagsFromPath, RenameFiles, TrackNumbers]:
-            nb.append_page(Page(self, self.__library))
+            page = Page(self, self.__library)
+            page.show()
+            nb.append_page(page)
         align = Alignment(nb, top=3)
         align.show()
         hp.pack2(align, resize=True, shrink=False)
         fs.connect('changed', self.__changed, l)
-        s = self.__library.connect_object('changed', FileSelector.rescan, fs)
+        if dir:
+            fs.go_to(dir)
+        s = self.__library.connect('changed', lambda *x: fs.rescan())
         self.connect_object('destroy', self.__library.disconnect, s)
         self.__save = None
         self.connect_object('changed', self.set_pending, None)
@@ -134,18 +148,20 @@ class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
             if resp == Gtk.ResponseType.YES:
                 self.__save.clicked()
             elif resp == Gtk.ResponseType.NO:
-                FileSelector.rescan(fs)
+                fs.rescan()
             else:
                 nb.grab_focus()
                 return True # cancel or closed
 
     def __popup_menu(self, view, fs):
-        view.grab_focus()
-        selection = view.get_selection()
-        model, rows = selection.get_selected_rows()
-        filenames = sorted([os.path.realpath(model[row][0]) for row in rows])
-        songs = map(self.__library.get, filenames)
-        if None not in songs:
+
+        # get all songs for the selection
+        filenames = fs.get_selected_paths()
+        filenames = map(normalize_path, map(os.path.realpath, filenames))
+        maybe_songs = [self.__library.get(f) for f in filenames]
+        songs = [s for s in maybe_songs if s]
+
+        if songs:
             menu = self.pm.Menu(self.__library, self, songs)
             if menu is None:
                 menu = Gtk.Menu()
@@ -153,19 +169,16 @@ class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
                 menu.prepend(SeparatorMenuItem())
         else:
             menu = Gtk.Menu()
-        b = Gtk.ImageMenuItem(Gtk.STOCK_DELETE, use_stock=True)
+
+        b = TrashMenuItem()
         b.connect('activate', self.__delete, filenames, fs)
         menu.prepend(b)
         menu.connect_object('selection-done', Gtk.Menu.destroy, menu)
         menu.show_all()
         return view.popup_menu(menu, 0, Gtk.get_current_event_time())
 
-    def __delete(self, item, files, fs):
-        d = DeleteDialog(self, files)
-        removed = d.run()
-        d.destroy()
-        removed = filter(None, map(self.__library.get, removed))
-        self.__library.remove(removed)
+    def __delete(self, item, paths, fs):
+        trash_files(self, paths)
         fs.rescan()
 
     def __changed(self, selector, selection, label):
@@ -184,7 +197,7 @@ class ExFalsoWindow(Gtk.Window, PersistentWindowMixin):
                 pass
             elif filename in self.__library:
                 file = self.__library[filename]
-                if file("~#mtime") + 1. < util.mtime(filename):
+                if file("~#mtime") + 1. < mtime(filename):
                     try:
                         file.reload()
                     except StandardError:
@@ -247,7 +260,7 @@ class PreferencesWindow(qltk.UniqueWindow):
         self.add(main_vbox)
 
         self.connect_object('destroy', PreferencesWindow.__destroy, self)
-        self.show_all()
+        self.get_child().show_all()
 
     def __changed(self, entry, section, name):
         config.set(section, name, entry.get_text())
