@@ -8,8 +8,7 @@
 
 import os
 
-from gi.repository import Gtk
-from gi.repository import Pango
+from gi.repository import Gtk, Gdk, Pango
 
 from quodlibet import const
 from quodlibet import qltk
@@ -20,18 +19,18 @@ from quodlibet.qltk.information import Information
 from quodlibet.qltk.ratingsmenu import RatingsMenuItem
 from quodlibet.qltk.x import SeparatorMenuItem
 
-from quodlibet.parse import XMLFromPattern
+from quodlibet.parse import XMLFromMarkupPattern
 from quodlibet.qltk.textedit import PatternEdit
 
 
-class SongInfo(Gtk.Label):
+class SongInfo(Gtk.EventBox):
     _pattern = ("""\
-\\<span weight='bold' size='large'\\><title>\\</span\\>\
+[span weight='bold' size='large']<title>[/span]\
 <~length| (<~length>)><version|
-\\<small\\>\\<b\\><version>\\</b\\>\\</small\\>><~people|
+[small][b]<version>[/b][/small]><~people|
 %(people)s><album|
-\\<b\\><album>\\</b\\><discnumber| - %(disc)s>\
-<discsubtitle| - \\<b\\><discsubtitle>\\</b\\>><tracknumber| - %(track)s>>"""
+[b]<album>[/b]<discnumber| - %(disc)s>\
+<discsubtitle| - [b]<discsubtitle>[/b]><tracknumber| - %(track)s>>"""
         % {
         # Translators: As in "by Artist Name"
         "people": _("by %s") % "<~people>",
@@ -43,19 +42,37 @@ class SongInfo(Gtk.Label):
 
     def __init__(self, library, player):
         super(SongInfo, self).__init__()
-        self.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
-        self.set_selectable(True)
-        self.set_alignment(0.0, 0.0)
+        self.set_visible_window(False)
+        align = Gtk.Alignment(xscale=0.0, xalign=0.0, yscale=0.0, yalign=0.0)
+        label = Gtk.Label()
+        label.set_ellipsize(Pango.EllipsizeMode.MIDDLE)
+        label.set_selectable(True)
+        align.add(label)
+        label.set_alignment(0.0, 0.0)
+        self._label = label
         library.connect_object('changed', self.__check_change, player)
         player.connect('song-started', self.__check_started)
 
-        self.connect_object('populate-popup', self.__menu, player, library)
+        label.connect_object('populate-popup', self.__menu, player, library)
+        self.connect('button-press-event', self.__create_menu, player, library)
 
         try:
             self._pattern = file(self.__PATTERN_FILENAME).read().rstrip()
         except EnvironmentError:
             pass
-        self._compiled = XMLFromPattern(self._pattern)
+        self._compiled = XMLFromMarkupPattern(self._pattern)
+        align.show_all()
+        self.add(align)
+
+    def __create_menu(self, widget, event, player, library):
+        button = event.button
+        if button == Gdk.BUTTON_SECONDARY:
+            menu = Gtk.Menu()
+            menu.attach_to_widget(widget, None)
+            self.__menu(player, menu, library)
+            menu.popup(None, None, None, None, button, event.time)
+            return True
+        return False
 
     def __menu(self, player, menu, library):
         try:
@@ -93,14 +110,20 @@ class SongInfo(Gtk.Label):
         menu.append(sep)
         sep.show()
         props = qltk.MenuItem(_("Edit _Tags"), Gtk.STOCK_PROPERTIES)
-        props.connect_object('activate', SongProperties, library,
-                             [player.info], self)
+
+        def song_properties_cb(menu_item):
+            window = SongProperties(library, [player.info], self)
+            window.show()
+        props.connect('activate', song_properties_cb)
         props.show()
         props.set_sensitive(bool(player.song))
         menu.append(props)
         info = Gtk.ImageMenuItem(Gtk.STOCK_INFO, use_stock=True)
-        info.connect_object('activate', Information, library,
-                            [player.info], self)
+
+        def information_cb(menu_item):
+            window = Information(library, [player.info], self)
+            window.show()
+        info.connect('activate', information_cb)
         info.show()
         menu.append(info)
         info.set_sensitive(bool(player.song))
@@ -109,6 +132,7 @@ class SongInfo(Gtk.Label):
         editor = PatternEdit(self, SongInfo._pattern)
         editor.text = self._pattern
         editor.apply.connect_object('clicked', self.__set, editor, player)
+        editor.show()
 
     def __set(self, edit, player):
         self._pattern = edit.text.rstrip()
@@ -121,7 +145,7 @@ class SongInfo(Gtk.Label):
             pattern_file = file(os.path.join(const.USERDIR, "songinfo"), "w")
             pattern_file.write(self._pattern + "\n")
             pattern_file.close()
-        self._compiled = XMLFromPattern(self._pattern)
+        self._compiled = XMLFromMarkupPattern(self._pattern)
         self.__update_info(player)
 
     def __check_change(self, player, songs):
@@ -140,6 +164,6 @@ class SongInfo(Gtk.Label):
         # some radio streams update way too often and updating the label
         # destroys the text selection
         if text not in last:
-            self.set_markup(text)
+            self._label.set_markup(text)
             last.clear()
             last[text] = True
