@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
-# Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna,
-#                     Steven Robertson, Nick Boultbee
+# Copyright 2004-2009 Joe Wreschnig, Michael Urman, Iñigo Serna,
+#                     Steven Robertson
+#           2011-2013 Nick Boultbee
 #           2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
@@ -14,17 +15,19 @@ from quodlibet import const
 from quodlibet import qltk
 from quodlibet import util
 from quodlibet import app
+from quodlibet.config import RatingsPrefs, RATINGS
 
 from quodlibet.parse import Query
-from quodlibet.qltk.ccb import ConfigCheckButton
+from quodlibet.qltk.ccb import ConfigCheckButton as CCB
 from quodlibet.qltk.data_editors import MultiStringEditor
 from quodlibet.qltk.entry import ValidatingEntry, UndoEntry
 from quodlibet.qltk.scanbox import ScanBox
 from quodlibet.qltk.maskedbox import MaskedBox
 from quodlibet.qltk.songlist import SongList
 from quodlibet.util import copool
+from quodlibet.util.path import fsnative
 from quodlibet.util.dprint import print_d
-from quodlibet.util.library import emit_signal
+from quodlibet.util.library import emit_signal, get_scan_dirs, scan_library
 
 
 class PreferencesWindow(qltk.UniqueWindow):
@@ -58,10 +61,10 @@ class PreferencesWindow(qltk.UniqueWindow):
 
             # Behaviour
             vbox = Gtk.VBox(spacing=6)
-            c = ConfigCheckButton(_("_Jump to playing song automatically"),
-                                  'settings', 'jump', populate=True)
-            c.set_tooltip_text(_("When the playing song changes, "
-                                 "scroll to it in the song list"))
+            c = CCB(_("_Jump to playing song automatically"),
+                    'settings', 'jump', populate=True,
+                    tooltip=_("When the playing song changes, "
+                              "scroll to it in the song list"))
             vbox.pack_start(c, False, True, 0)
             frame = qltk.Frame(_("Behavior"), child=vbox)
             self.pack_start(frame, False, True, 0)
@@ -69,13 +72,12 @@ class PreferencesWindow(qltk.UniqueWindow):
             # Columns
             vbox = Gtk.VBox(spacing=12)
             buttons = {}
-            table = Gtk.Table(3, 3)
-            table.set_homogeneous(True)
+            table = Gtk.Table.new(3, 3, True)
             cols = config.get_columns(refresh=True)
 
             for i, (k, t) in enumerate(self.PREDEFINED_TAGS):
                 x, y = i % 3, i / 3
-                buttons[k] = Gtk.CheckButton(t, use_underline=True)
+                buttons[k] = Gtk.CheckButton(label=t, use_underline=True)
                 if k in cols:
                     buttons[k].set_active(True)
                     cols.remove(k)
@@ -92,7 +94,7 @@ class PreferencesWindow(qltk.UniqueWindow):
             self.others = others = UndoEntry()
             others.set_sensitive(False)
             # Stock edit doesn't have ellipsis chars.
-            edit_button = Gtk.Button(_("_Edit..."), use_underline=True)
+            edit_button = Gtk.Button(label=_("_Edit..."), use_underline=True)
             edit_button.connect("clicked", self.__config_cols)
             edit_button.set_tooltip_text(_("Add or remove additional column "
                                            "headers"))
@@ -106,13 +108,13 @@ class PreferencesWindow(qltk.UniqueWindow):
             self.pack_start(frame, False, True, 0)
 
             # Column preferences
-            tiv = Gtk.CheckButton(_("Title includes _version"),
+            tiv = Gtk.CheckButton(label=_("Title includes _version"),
                                   use_underline=True)
-            aio = Gtk.CheckButton(_("Artist includes all _people"),
+            aio = Gtk.CheckButton(label=_("Artist includes all _people"),
                                   use_underline=True)
-            aip = Gtk.CheckButton(_("Album includes _disc subtitle"),
+            aip = Gtk.CheckButton(label=_("Album includes _disc subtitle"),
                                   use_underline=True)
-            fip = Gtk.CheckButton(_("Filename includes _folder"),
+            fip = Gtk.CheckButton(label=_("Filename includes _folder"),
                                   use_underline=True)
             self._toggle_data = [
                 (tiv, "title", "~title~version"),
@@ -129,8 +131,7 @@ class PreferencesWindow(qltk.UniqueWindow):
 
             # Update text once to exclude ticked columns, munged or not
             others.set_text(", ".join(cols))
-            t = Gtk.Table(2, 2)
-            t.set_homogeneous(True)
+            t = Gtk.Table.new(2, 2, True)
             t.attach(tiv, 0, 1, 0, 1)
             t.attach(aip, 0, 1, 1, 2)
             t.attach(aio, 1, 2, 0, 1)
@@ -152,7 +153,9 @@ class PreferencesWindow(qltk.UniqueWindow):
                           aio)
             # Apply on destroy, else config gets mangled
             self.connect('destroy', self.__apply, buttons, tiv, aip, fip, aio)
-            self.show_all()
+
+            for child in self.get_children():
+                child.show_all()
 
         def __apply(self, button, buttons, tiv, aip, fip, aio):
             new_headers = set()
@@ -197,6 +200,7 @@ class PreferencesWindow(qltk.UniqueWindow):
 
             m = MultiStringEditor(_("Extra Columns"), self.other_cols)
             m.connect('destroy', __closed)
+            m.show()
 
     class Browsers(Gtk.VBox):
         name = "browser"
@@ -220,26 +224,21 @@ class PreferencesWindow(qltk.UniqueWindow):
             hb.pack_start(e, True, True, 0)
             vb.pack_start(hb, False, True, 0)
 
-            c = ConfigCheckButton(_("Search after _typing"),
-                                  'settings', 'eager_search', populate=True)
-            c.set_tooltip_text(
-                    _("Show search results after the user stops typing."))
-            vb.pack_start(c, False, True, 0)
             # Translators: The heading of the preference group, no action
             f = qltk.Frame(Q_("heading|Search"), child=vb)
             self.pack_start(f, False, True, 0)
 
             # Ratings
             vb = Gtk.VBox(spacing=6)
-            c1 = ConfigCheckButton(_("Confirm _multiple ratings"),
-                    'browsers', 'rating_confirm_multiple', populate=True)
-            c1.set_tooltip_text(_("Ask for confirmation before changing the "
-                                  "rating of multiple songs at once"))
+            c1 = CCB(_("Confirm _multiple ratings"),
+                     'browsers', 'rating_confirm_multiple', populate=True,
+                     tooltip=_("Ask for confirmation before changing the "
+                               "rating of multiple songs at once"))
 
-            c2 = ConfigCheckButton(_("Enable _one-click ratings"),
-                                   'browsers', 'rating_click', populate=True)
-            c2.set_tooltip_text(_("Enable rating by clicking on the rating "
-                                  "column in the song list"))
+            c2 = CCB(_("Enable _one-click ratings"),
+                     'browsers', 'rating_click', populate=True,
+                     tooltip=_("Enable rating by clicking on the rating "
+                               "column in the song list"))
 
             vbox = Gtk.VBox(spacing=6)
             vbox.pack_start(c1, False, True, 0)
@@ -249,24 +248,24 @@ class PreferencesWindow(qltk.UniqueWindow):
 
             # Album Art
             vb = Gtk.VBox(spacing=6)
-            c = ConfigCheckButton(_("_Use rounded corners on thumbnails"),
-                                  'albumart', 'round', populate=True)
-            c.set_tooltip_text(_("Round the corners of album artwork "
-                    "thumbnail images. May require restart to take effect."))
+            c = CCB(_("_Use rounded corners on thumbnails"),
+                    'albumart', 'round', populate=True,
+                    tooltip=_("Round the corners of album artwork thumbnail "
+                              "images. May require restart to take effect."))
             vb.pack_start(c, False, True, 0)
 
             # Filename choice algorithm config
-            cb = ConfigCheckButton(_("Prefer _embedded art"),
-                                   'albumart', 'prefer_embedded',
-                                   populate=True)
-            cb.set_tooltip_text(
-                _("Choose to use artwork embedded in the audio "
-                  "(where available) over other sources"))
+            cb = CCB(_("Prefer _embedded art"),
+                     'albumart', 'prefer_embedded', populate=True,
+                     tooltip=_("Choose to use artwork embedded in the audio "
+                               "(where available) over other sources"))
             vb.pack_start(cb, False, True, 0)
 
             hb = Gtk.HBox(spacing=3)
-            cb = ConfigCheckButton(_("_Force image filename:"),
-                                   'albumart', 'force_filename', populate=True)
+            cb = CCB(_("_Fixed image filename:"),
+                     'albumart', 'force_filename', populate=True,
+                     tooltip=_("The single image filename to use if "
+                               "selected"))
             hb.pack_start(cb, False, True, 0)
 
             entry = UndoEntry()
@@ -283,6 +282,9 @@ class PreferencesWindow(qltk.UniqueWindow):
             f = qltk.Frame(_("Album Art"), child=vb)
             self.pack_start(f, False, True, 0)
 
+            for child in self.get_children():
+                child.show_all()
+
         def __changed_text(self, entry, name):
             config.set('albumart', name, entry.get_text())
 
@@ -308,7 +310,7 @@ class PreferencesWindow(qltk.UniqueWindow):
 
             # replaygain
             fallback_gain = config.getfloat("player", "fallback_gain", 0.0)
-            adj = Gtk.Adjustment(fallback_gain, -12.0, 12.0, 0.5, 0.5, 0.0)
+            adj = Gtk.Adjustment.new(fallback_gain, -12.0, 12.0, 0.5, 0.5, 0.0)
             fb_spin = Gtk.SpinButton(adjustment=adj)
             fb_spin.set_digits(1)
             fb_spin.connect('changed', self.__changed,
@@ -322,7 +324,7 @@ class PreferencesWindow(qltk.UniqueWindow):
             fb_label.set_mnemonic_widget(fb_spin)
 
             pre_amp_gain = config.getfloat("player", "pre_amp_gain", 0.0)
-            adj = Gtk.Adjustment(pre_amp_gain, -6, 6, 0.5, 0.5, 0.0)
+            adj = Gtk.Adjustment.new(pre_amp_gain, -6, 6, 0.5, 0.5, 0.0)
             adj.connect('value-changed', self.__changed,
                         'player', 'pre_amp_gain')
             pre_spin = Gtk.SpinButton(adjustment=adj)
@@ -336,12 +338,12 @@ class PreferencesWindow(qltk.UniqueWindow):
             pre_label.set_mnemonic_widget(pre_spin)
 
             widgets = [pre_label, pre_spin, fb_label, fb_spin]
-            c = ConfigCheckButton(_("_Enable Replay Gain volume adjustment"),
-                                    "player", "replaygain", populate=True)
+            c = CCB(_("_Enable Replay Gain volume adjustment"),
+                    "player", "replaygain", populate=True)
             c.connect('toggled', self.__toggled_gain, widgets)
 
             # packing
-            table = Gtk.Table(3, 2)
+            table = Gtk.Table.new(3, 2, False)
             table.set_col_spacings(6)
             table.set_row_spacings(6)
 
@@ -366,7 +368,9 @@ class PreferencesWindow(qltk.UniqueWindow):
             c.emit('toggled')
 
             self.pack_start(f, False, True, 0)
-            self.show_all()
+
+            for child in self.get_children():
+                child.show_all()
 
         def __toggled_gain(self, activator, widgets):
             if app.player: # tests
@@ -381,48 +385,102 @@ class PreferencesWindow(qltk.UniqueWindow):
     class Tagging(Gtk.VBox):
         name = "tagging"
 
-        def __init__(self):
-            super(PreferencesWindow.Tagging, self).__init__(spacing=12)
-            self.set_border_width(12)
-            self.title = _("Tags")
-            self._songs = []
+        def ratings_vbox(self):
+            """Returns a new VBox containing all ratings widgets"""
+            vb = Gtk.VBox(spacing=6)
 
-            vbox = Gtk.VBox(spacing=6)
+            # Default Rating
+            model = Gtk.ListStore(float)
+            default_combo = Gtk.ComboBox(model=model)
+            default_lab = Gtk.Label(label=_("_Default rating:"))
+            default_lab.set_use_underline(True)
+            default_lab.set_alignment(0, 0.5)
 
-            cb = ConfigCheckButton(_("Auto-save tag changes"),
-                                   'editing', 'auto_save_changes',
-                                   populate=True)
-            cb.set_tooltip_text(_("Save changes to tags without confirmation "
-                                  "when editing multiple files"))
-            vbox.pack_start(cb, False, True, 0)
+            def draw_rating(column, cell, model, it, data):
+                num = model[it][0]
+                text = "%0.2f: %s" % (num, util.format_rating(num))
+                cell.set_property('text', text)
 
-            cb = ConfigCheckButton(_("Show _programmatic tags"),
-                                   'editing', 'alltags', populate=True)
-            cb.set_tooltip_text(
-                    _("Access all tags, including machine-generated ones "
-                      "e.g. MusicBrainz or Replay Gain tags"))
-            vbox.pack_start(cb, False, True, 0)
+            def default_rating_changed(combo, model):
+                it = combo.get_active_iter()
+                if it is None:
+                    return
+                RATINGS.default = model[it][0]
 
-            hb = Gtk.HBox(spacing=6)
-            e = UndoEntry()
-            e.set_text(config.get("editing", "split_on"))
-            e.connect('changed', self.__changed, 'editing', 'split_on')
-            e.set_tooltip_text(
-                    _("A list of separators to use when splitting tag values. "
-                      "The list is space-separated"))
-            l = Gtk.Label(label=_("Split _on:"))
-            l.set_use_underline(True)
-            l.set_mnemonic_widget(e)
-            hb.pack_start(l, False, True, 0)
-            hb.pack_start(e, True, True, 0)
-            vbox.pack_start(hb, False, True, 0)
+            def populate_default_rating_model(combo, num):
+                model = combo.get_model()
+                model.clear()
+                deltas = []
+                default = RATINGS.default
+                precision = RATINGS.precision
+                for i in range(0, num + 1):
+                    r = i * precision
+                    model.append(row=[r])
+                    deltas.append((abs(default - r), i))
+                active = sorted(deltas)[0][1]
+                print_d("Choosing #%d (%.2f), closest to current %.2f"
+                        % (active, precision * active, default))
+                combo.set_active(active)
 
-            vb2 = Gtk.VBox(spacing=6)
+            cell = Gtk.CellRendererText()
+            default_combo.pack_start(cell, True)
+            default_combo.set_cell_data_func(cell, draw_rating, None)
+            default_combo.connect('changed', default_rating_changed, model)
+            default_lab.set_mnemonic_widget(default_combo)
 
+            def refresh_default_combo(num):
+                populate_default_rating_model(default_combo, num)
+
+            # Rating Scale
+            model = Gtk.ListStore(int)
+            scale_combo = Gtk.ComboBox(model=model)
+            scale_lab = Gtk.Label(label=_("Rating _Scale:"))
+            scale_lab.set_use_underline(True)
+            scale_lab.set_mnemonic_widget(scale_combo)
+
+            cell = Gtk.CellRendererText()
+            scale_combo.pack_start(cell, False)
+            num = RATINGS.number
+            for i in [1, 2, 3, 4, 5, 6, 8, 10]:
+                it = model.append(row=[i])
+                if i == num:
+                    scale_combo.set_active_iter(it)
+
+            def draw_rating_scale(column, cell, model, it, data):
+                num_stars = model[it][0]
+                text = "%d: %s" % (num_stars, RATINGS.full_symbol * num_stars)
+                cell.set_property('text', text)
+
+            def rating_scale_changed(combo, model):
+                it = combo.get_active_iter()
+                if it is None:
+                    return
+                RATINGS.number = num = model[it][0]
+                refresh_default_combo(num)
+
+            refresh_default_combo(RATINGS.number)
+            scale_combo.set_cell_data_func(cell, draw_rating_scale, None)
+            scale_combo.connect('changed', rating_scale_changed, model)
+
+            default_align = Gtk.Alignment(xalign=0, xscale=0)
+            default_align.add(default_lab)
+            default_combo_align = Gtk.Alignment(xalign=0, xscale=0)
+            default_combo_align.add(default_combo)
+            scale_align = Gtk.Alignment(xalign=0, xscale=0)
+            scale_align.add(scale_lab)
+
+            grid = Gtk.Grid(column_spacing=6, row_spacing=6)
+            grid.add(scale_align)
+            grid.add(scale_combo)
+            grid.attach(default_align, 0, 1, 1, 1)
+            grid.attach(default_combo_align, 1, 1, 1, 1)
+            vb.pack_start(grid, False, False, 6)
+
+            # Bayesian Factor
             bayesian_factor = config.getfloat("settings",
                                               "bayesian_rating_factor", 0.0)
-            adj = Gtk.Adjustment(bayesian_factor, 0.0, 10.0, 0.5, 0.5, 0.0)
-            bayes_spin = Gtk.SpinButton(adjustment=adj)
+            adj = Gtk.Adjustment.new(bayesian_factor, 0.0, 10.0, 0.5, 0.5, 0.0)
+            bayes_spin = Gtk.SpinButton(adjustment=adj, numeric=True)
             bayes_spin.set_digits(1)
             bayes_spin.connect('changed', self.__changed_and_signal_library,
                                'settings', 'bayesian_rating_factor')
@@ -432,18 +490,18 @@ class PreferencesWindow(qltk.UniqueWindow):
                   "albums with few tracks will have less extreme ratings. "
                   "Changing this value triggers a re-calculation for all "
                   "albums."))
-
             bayes_label = Gtk.Label(label=_("_Bayesian averaging amount:"))
             bayes_label.set_use_underline(True)
             bayes_label.set_mnemonic_widget(bayes_spin)
+
+            # Save Ratings
             hb = Gtk.HBox(spacing=6)
             hb.pack_start(bayes_label, False, True, 0)
             hb.pack_start(bayes_spin, False, True, 0)
-            vb2.pack_start(hb, True, True, 0)
-
-            cb = ConfigCheckButton(_("Save ratings and play _counts"),
-                                   "editing", "save_to_songs", populate=True)
-            vb2.pack_start(cb, True, True, 0)
+            vb.pack_start(hb, True, True, 0)
+            cb = CCB(_("Save ratings and play _counts"),
+                     "editing", "save_to_songs", populate=True)
+            vb.pack_start(cb, True, True, 0)
             hb = Gtk.HBox(spacing=6)
             lab = Gtk.Label(label=_("_Email:"))
             entry = UndoEntry()
@@ -455,21 +513,69 @@ class PreferencesWindow(qltk.UniqueWindow):
             hb.pack_start(entry, True, True, 0)
             lab.set_mnemonic_widget(entry)
             lab.set_use_underline(True)
-            vb2.pack_start(hb, True, True, 0)
+            vb.pack_start(hb, True, True, 0)
 
-            f = qltk.Frame(_("Tag Editing"), child=vbox)
+            return vb
+
+        def tag_editing_vbox(self):
+            """Returns a new VBox containing all tag editing widgets"""
+            vbox = Gtk.VBox(spacing=6)
+            cb = CCB(_("Auto-save tag changes"), 'editing',
+                     'auto_save_changes', populate=True,
+                     tooltip=_("Save changes to tags without confirmation "
+                               "when editing multiple files"))
+            vbox.pack_start(cb, False, True, 0)
+            cb = CCB(_("Show _programmatic tags"),
+                     'editing', 'alltags', populate=True,
+                     tooltip=_("Access all tags, including machine-generated "
+                               "ones e.g. MusicBrainz or Replay Gain tags"))
+            vbox.pack_start(cb, False, True, 0)
+            hb = Gtk.HBox(spacing=6)
+            e = UndoEntry()
+            e.set_text(config.get("editing", "split_on"))
+            e.connect('changed', self.__changed, 'editing', 'split_on')
+            e.set_tooltip_text(
+                _("A list of separators to use when splitting tag values. "
+                  "The list is space-separated"))
+
+            def do_revert_split(button, section, option):
+                config.reset(section, option)
+                e.set_text(config.get(section, option))
+
+            split_revert = Gtk.Button()
+            split_revert.add(Gtk.Image.new_from_stock(
+                             Gtk.STOCK_REVERT_TO_SAVED, Gtk.IconSize.MENU))
+            split_revert.connect("clicked", do_revert_split, "editing",
+                                 "split_on")
+            l = Gtk.Label(label=_("Split _on:"))
+            l.set_use_underline(True)
+            l.set_mnemonic_widget(e)
+            hb.pack_start(l, False, True, 0)
+            hb.pack_start(e, True, True, 0)
+            hb.pack_start(split_revert, False, True, 0)
+            vbox.pack_start(hb, False, True, 0)
+            return vbox
+
+        def __init__(self):
+            super(PreferencesWindow.Tagging, self).__init__(spacing=12)
+            self.set_border_width(12)
+            self.title = _("Tags")
+            self._songs = []
+
+            f = qltk.Frame(_("Tag Editing"), child=(self.tag_editing_vbox()))
             self.pack_start(f, False, True, 0)
 
-            f = qltk.Frame(_("Ratings"), child=vb2)
+            f = qltk.Frame(_("Ratings"), child=self.ratings_vbox())
             self.pack_start(f, False, True, 0)
 
-            self.show_all()
+            for child in self.get_children():
+                child.show_all()
 
         def __changed(self, entry, section, name):
             config.set(section, name, entry.get_text())
 
         def __changed_and_signal_library(self, entry, section, name):
-            self.__changed(entry, section, name)
+            config.set(section, name, str(entry.get_value()))
             print_d("Signalling \"changed\" to entire library. Hold tight...")
             # Cache over clicks
             self._songs = self._songs or app.library.values()
@@ -484,28 +590,36 @@ class PreferencesWindow(qltk.UniqueWindow):
             self.set_border_width(12)
             self.title = _("Library")
 
-            cb = ConfigCheckButton(_("_Refresh library on start"),
-                                   "library", "refresh_on_start",
-                                   populate=True)
+            cb = CCB(_("_Refresh library on start"),
+                     "library", "refresh_on_start", populate=True)
             scan_dirs = ScanBox()
 
             vb3 = Gtk.VBox(spacing=6)
             vb3.pack_start(scan_dirs, True, True, 0)
 
             def refresh_cb(button):
-                paths = util.split_scan_dirs(config.get("settings", "scan"))
-                exclude = config.get("library", "exclude").split(":")
-                copool.add(app.library.rebuild, paths, False, exclude,
-                           cofuncid="library", funcid="library")
+                scan_library(app.library, force=False)
 
-            refresh = qltk.Button(_("Refresh Library"), Gtk.STOCK_REFRESH)
+            refresh = qltk.Button(_("Re_fresh Library"), Gtk.STOCK_REFRESH)
             refresh.connect("clicked", refresh_cb)
+            refresh.set_tooltip_text(_("Check for changes in your library"))
 
-            hb = Gtk.HBox(spacing=6)
-            hb.pack_start(cb, True, True, 0)
-            hb.pack_start(refresh, False, True, 0)
+            def reload_cb(button):
+                scan_library(app.library, force=True)
 
-            vb3.pack_start(hb, False, True, 0)
+            reload_ = qltk.Button(_("Re_load Library"), Gtk.STOCK_REFRESH)
+            reload_.connect("clicked", reload_cb)
+            reload_.set_tooltip_text(
+                _("Reload all songs in your library. "
+                  "(this can take a long time)"))
+
+            grid = Gtk.Grid(column_spacing=6, row_spacing=6)
+            cb.props.hexpand = True
+            grid.attach(cb, 0, 0, 1, 1)
+            grid.attach(refresh, 1, 0, 1, 1)
+            grid.attach(reload_, 1, 1, 1, 1)
+
+            vb3.pack_start(grid, False, True, 0)
             f = qltk.Frame(_("Scan Directories"), child=vb3)
             self.pack_start(f, False, True, 0)
 
@@ -515,12 +629,14 @@ class PreferencesWindow(qltk.UniqueWindow):
                 f = qltk.Frame(_("Hidden Songs"), child=masked)
                 self.pack_start(f, False, True, 0)
 
-            self.show_all()
+            for child in self.get_children():
+                child.show_all()
 
     def __init__(self, parent):
         if self.is_not_unique():
             return
         super(PreferencesWindow, self).__init__()
+        self.current_scan_dirs = get_scan_dirs()
         self.set_title(_("Preferences") + " - Quod Libet")
         self.set_border_width(12)
         self.set_resizable(False)
@@ -543,7 +659,8 @@ class PreferencesWindow(qltk.UniqueWindow):
         self.add(vbox)
 
         self.connect_object('destroy', PreferencesWindow.__destroy, self)
-        self.show_all()
+
+        self.get_child().show_all()
 
     def set_page(self, name):
         notebook = self.__notebook
@@ -553,3 +670,5 @@ class PreferencesWindow(qltk.UniqueWindow):
 
     def __destroy(self):
         config.write(const.CONFIG)
+        if self.current_scan_dirs != get_scan_dirs():
+            scan_library(app.library, force=False)
