@@ -9,7 +9,6 @@
 from gi.repository import Gtk, GObject, Pango
 
 from quodlibet import qltk
-from quodlibet import util
 from quodlibet import config
 
 from quodlibet.qltk.edittags import EditTags
@@ -18,6 +17,7 @@ from quodlibet.qltk.tagsfrompath import TagsFromPath
 from quodlibet.qltk.tracknumbers import TrackNumbers
 from quodlibet.qltk.views import HintedTreeView
 from quodlibet.qltk.window import PersistentWindowMixin
+from quodlibet.util.path import fsdecode
 
 
 class SongProperties(qltk.Window, PersistentWindowMixin):
@@ -50,6 +50,7 @@ class SongProperties(qltk.Window, PersistentWindowMixin):
         if len(songs) > 1:
             pages.append(TrackNumbers(self, library))
         for page in pages:
+            page.show()
             notebook.append_page(page)
         self.set_border_width(12)
 
@@ -80,7 +81,7 @@ class SongProperties(qltk.Window, PersistentWindowMixin):
         # checks that "hardcode" the first row being selected.
 
         for song in songs:
-            fbasemodel.append(row=[song, util.fsdecode(song("~basename"))])
+            fbasemodel.append(row=[song, fsdecode(song("~basename"))])
 
         self.connect_object('changed', SongProperties.__set_title, self)
 
@@ -101,7 +102,6 @@ class SongProperties(qltk.Window, PersistentWindowMixin):
         paned.set_position(175)
         notebook.show()
         paned.show()
-        self.show()
 
     def __remove(self, library, songs, model, selection, sig):
         # If the handler is unblocked, then the selection gets updated
@@ -121,7 +121,8 @@ class SongProperties(qltk.Window, PersistentWindowMixin):
             if row[0] in songs:
                 to_remove.append(row.iter)
                 changed = changed or (row.path in rows)
-        map(model.remove, to_remove)
+        for iter_ in to_remove:
+            model.remove(iter_)
         selection.handler_unblock(sig)
         if changed:
             selection.emit('changed')
