@@ -7,10 +7,11 @@
 # published by the Free Software Foundation
 
 import os
-import sys
 
 from gi.repository import Gtk, GObject, Gdk, GLib, Gio
 
+import quodlibet
+
 from quodlibet import browsers
 from quodlibet import config
 from quodlibet import const
@@ -25,6 +26,7 @@ from quodlibet.qltk.chooser import FolderChooser, FileChooser
 from quodlibet.qltk.controls import PlayControls
 from quodlibet.qltk.cover import CoverImage
 from quodlibet.qltk.getstring import GetStringDialog
+from quodlibet.qltk.bookmarks import EditBookmarks
 from quodlibet.qltk.info import SongInfo
 from quodlibet.qltk.information import Information
 from quodlibet.qltk.logging import LoggingWindow
@@ -40,8 +42,9 @@ from quodlibet.qltk.songmodel import PlaylistMux
 from quodlibet.qltk.x import RPaned, ConfigRVPaned, Alignment, ScrolledWindow
 from quodlibet.qltk.about import AboutQuodLibet
 from quodlibet.util import copool, gobject_weak
+from quodlibet.util.library import get_scan_dirs, set_scan_dirs
 from quodlibet.util.uri import URI
-from quodlibet.util.library import background_filter, scan_libary
+from quodlibet.util.library import background_filter, scan_library
 from quodlibet.qltk.window import PersistentWindowMixin
 
 
@@ -122,7 +125,7 @@ class MainSongList(SongList):
 class SongListScroller(ScrolledWindow):
     def __init__(self, menu):
         super(SongListScroller, self).__init__()
-        self.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.ALWAYS)
+        self.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         self.set_shadow_type(Gtk.ShadowType.IN)
         self.connect_object('notify::visible', self.__visibility, menu)
 
@@ -132,18 +135,29 @@ class SongListScroller(ScrolledWindow):
         config.set("memory", "songlist", str(value))
 
 
-class TopBar(Gtk.HBox):
+class TopBar(Gtk.Toolbar):
     def __init__(self, parent, player, library):
-        super(TopBar, self).__init__(spacing=3)
+        super(TopBar, self).__init__()
 
         # play controls
+        control_item = Gtk.ToolItem()
+        self.insert(control_item, 0)
         t = PlayControls(player, library.librarian)
         self.volume = t.volume
-        self.pack_start(t, False, False, 0)
+        control_item.add(t)
+
+        self.insert(Gtk.SeparatorToolItem(), 1)
+
+        info_item = Gtk.ToolItem()
+        self.insert(info_item, 2)
+        info_item.set_expand(True)
+
+        box = Gtk.Box(spacing=6)
+        info_item.add(Alignment(box, border=3, right=-3))
 
         # song text
         text = SongInfo(library.librarian, player)
-        self.pack_start(Alignment(text, border=3), True, True, 0)
+        box.pack_start(Alignment(text, top=3, bottom=3), True, True, 0)
 
         # cover image
         self.image = CoverImage(resize=True)
@@ -151,7 +165,13 @@ class TopBar(Gtk.HBox):
                      parent=self)
         gobject_weak(parent.connect, 'artwork-changed',
                      self.__song_art_changed, library, parent=self)
-        self.pack_start(self.image, False, True, 0)
+        box.pack_start(self.image, False, True, 0)
+
+        for child in self.get_children():
+            child.show_all()
+
+        context = self.get_style_context()
+        context.add_class("primary-toolbar")
 
     def __new_song(self, player, song):
         self.image.set_song(song)
@@ -198,6 +218,96 @@ class StatusBarBox(Gtk.HBox):
         model.repeat = button.get_active()
 
 
+class AppMenu(object):
+    """Implements a app menu proxy mirroring some main menu items
+    to a new menu and exporting it on the session bus.
+
+    Activation gets proxied back to the main menu actions.
+    """
+
+    def __init__(self, window, action_group):
+        window.realize()
+
+        self._bus = None
+        self._ag_id = None
+        self._am_id = None
+        window.connect("destroy", self._unexport)
+
+        if window.get_realized():
+            self._export(window, action_group)
+        else:
+            self._id = window.connect("realize", self._realized, action_group)
+
+    def _realized(self, window, ag):
+        window.disconnect(self._id)
+        self._export(window, ag)
+
+    def _export(self, window, gtk_group):
+        actions = [
+            ["Preferences", "Plugins"],
+            ["RefreshLibrary"],
+            ["OnlineHelp", "About", "Quit"],
+        ]
+
+        # build the new menu
+        menu = Gio.Menu()
+        action_names = []
+        for group in actions:
+            section = Gio.Menu()
+            for name in group:
+                action = gtk_group.get_action(name)
+                assert action
+                label = action.get_label()
+                section.append(label, "app." + name)
+                action_names.append(name)
+            menu.append_section(None, section)
+        menu.freeze()
+
+        # proxy activate to the old group
+        def callback(action, data):
+            name = action.get_name()
+            gtk_action = gtk_group.get_action(name)
+            gtk_action.activate()
+
+        action_group = Gio.SimpleActionGroup()
+        for name in action_names:
+            action = Gio.SimpleAction.new(name, None)
+            action_group.insert(action)
+            action.connect("activate", callback)
+
+        # export on the bus
+        ag_object_path = "/net/sacredchao/QuodLibet"
+        am_object_path = "/net/sacredchao/QuodLibet/menus/appmenu"
+        app_id = "net.sacredchao.QuodLibet"
+
+        try:
+            bus = Gio.bus_get_sync(Gio.BusType.SESSION, None)
+            self._ag_id = bus.export_action_group(ag_object_path, action_group)
+            self._am_id = bus.export_menu_model(am_object_path, menu)
+        except GLib.GError as e:
+            print_d("Registering appmenu failed: %r" % e)
+            return
+
+        self._bus = bus
+
+        win = window.get_window()
+        if not hasattr(win, "set_utf8_property"):
+            # not a GdkX11.X11Window
+            print_d("Registering appmenu failed: X11 only")
+            return
+
+        win.set_utf8_property("_GTK_UNIQUE_BUS_NAME", bus.get_unique_name())
+        win.set_utf8_property("_GTK_APPLICATION_ID", app_id)
+        win.set_utf8_property("_GTK_APPLICATION_OBJECT_PATH", ag_object_path)
+        win.set_utf8_property("_GTK_APP_MENU_OBJECT_PATH", am_object_path)
+
+    def _unexport(self, window):
+        if self._bus:
+            self._bus.unexport_action_group(self._ag_id)
+            self._bus.unexport_menu_model(self._am_id)
+            self._bus = None
+
+
 DND_URI_LIST, = range(1)
 
 
@@ -207,7 +317,7 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
         'artwork-changed': SIG_PYOBJECT,
     }
 
-    def __init__(self, library, player):
+    def __init__(self, library, player, headless=False):
         super(QuodLibetWindow, self).__init__()
         self.last_dir = const.HOME
 
@@ -219,35 +329,39 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
 
         # create main menubar, load/restore accelerator groups
         self.__library = library
-        self.__create_menu(player, library)
-        self.add_accel_group(self.ui.get_accel_group())
+        ui = self.__create_menu(player, library)
+        accel_group = ui.get_accel_group()
+        self.add_accel_group(accel_group)
+
+        # dbus app menu
+        AppMenu(self, ui.get_action_groups()[0])
 
         accel_fn = os.path.join(const.USERDIR, "accels")
         Gtk.AccelMap.load(accel_fn)
-        accelgroup = Gtk.accel_groups_from_object(self)[0]
-        GObject.Object.connect(accelgroup, 'accel-changed',
-                lambda *args: Gtk.AccelMap.save(accel_fn))
-        main_box.pack_start(self.ui.get_widget("/Menu"), False, True, 0)
+
+        def accel_save_cb(*args):
+            Gtk.AccelMap.save(accel_fn)
+        accel_group.connect_object('accel-changed', accel_save_cb, None)
+        main_box.pack_start(ui.get_widget("/Menu"), False, True, 0)
 
         # get the playlist up before other stuff
         self.songlist = MainSongList(library, player)
         self.songlist.show_all()
-        self.add_accel_group(self.songlist.accelerators)
         self.songlist.connect_after(
             'drag-data-received', self.__songlist_drag_data_recv)
         self.song_scroller = SongListScroller(
-            self.ui.get_widget("/Menu/View/SongList"))
+            ui.get_widget("/Menu/View/SongList"))
         self.song_scroller.add(self.songlist)
         self.qexpander = QueueExpander(
-            self.ui.get_widget("/Menu/View/Queue"), library, player)
+            ui.get_widget("/Menu/View/Queue"), library, player)
         self.playlist = PlaylistMux(
             player, self.qexpander.model, self.songlist.model)
 
         top_bar = TopBar(self, player, library)
-        top_align = Alignment(top_bar, border=3, bottom=-3)
-        main_box.pack_start(top_align, False, True, 0)
+        main_box.pack_start(top_bar, False, True, 0)
+        self.top_bar = top_bar
 
-        self.__browserbox = Alignment(top=3, bottom=3)
+        self.__browserbox = Alignment(bottom=3)
         main_box.pack_start(self.__browserbox, True, True, 0)
 
         statusbox = StatusBarBox(self.songlist.model, player)
@@ -255,8 +369,9 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
         self.repeat = statusbox.repeat
         self.statusbar = statusbox.statusbar
 
-        main_box.pack_start(Alignment(statusbox, border=3, top=-3),
-                            False, True, 0)
+        main_box.pack_start(
+            Alignment(statusbox, border=3, top=-3, left=6, right=6),
+            False, True, 0)
 
         self.songpane = ConfigRVPaned("memory", "queue_position", 0.75)
         self.songpane.pack1(self.song_scroller, resize=True, shrink=False)
@@ -273,6 +388,7 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
         self.songlist.set_sort_by(None, sort[1:], order=int(sort[0]))
 
         self.browser = None
+        self.ui = ui
 
         main_box.show_all()
 
@@ -288,13 +404,14 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
         player.setup(self.playlist, None, 0)
 
         def delayed_song_set():
+            self.__delayed_setup = None
             song = library.get(config.get("memory", "song"))
-            seek_pos = config.getint("memory", "seek")
+            seek_pos = config.getint("memory", "seek", 0)
             config.set("memory", "seek", 0)
             player.setup(self.playlist, song, seek_pos)
         self.__delayed_setup = GLib.idle_add(delayed_song_set)
-        self.showhide_playlist(self.ui.get_widget("/Menu/View/SongList"))
-        self.showhide_playqueue(self.ui.get_widget("/Menu/View/Queue"))
+        self.showhide_playlist(ui.get_widget("/Menu/View/SongList"))
+        self.showhide_playqueue(ui.get_widget("/Menu/View/Queue"))
 
         self.songlist.connect('popup-menu', self.__songs_popup_menu)
         self.songlist.connect('columns-changed', self.__cols_changed)
@@ -306,7 +423,6 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
                      player, parent=self)
 
         player_sigs = [
-            ('song-ended', self.__song_ended),
             ('song-started', self.__song_started),
             ('paused', self.__update_paused, True),
             ('unpaused', self.__update_paused, False),
@@ -314,6 +430,11 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
         for sig in player_sigs:
             gobject_weak(player.connect, *sig, **{"parent": self})
 
+        # connect after to let SongTracker update stats
+        player_sigs.append(
+            gobject_weak(player.connect_after, "song-ended",
+                         self.__song_ended, parent=self))
+
         targets = [("text/uri-list", Gtk.TargetFlags.OTHER_APP, DND_URI_LIST)]
         targets = [Gtk.TargetEntry.new(*t) for t in targets]
 
@@ -321,6 +442,9 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
             Gtk.DestDefaults.ALL, targets, Gdk.DragAction.COPY)
         self.connect('drag-data-received', self.__drag_data_received)
 
+        if not headless:
+            GLib.idle_add(self.__configure_scan_dirs, library)
+
         if config.getboolean('library', 'refresh_on_start'):
             self.__rebuild(None, False)
 
@@ -331,6 +455,32 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
 
         self.enable_window_tracking("quodlibet")
 
+    def __configure_scan_dirs(self, library):
+        """Get user to configure scan dirs, if none is set up"""
+        if not get_scan_dirs() and not len(library) and \
+                quodlibet.is_first_session("quodlibet"):
+            print_d("Couldn't find any scan dirs")
+            if qltk.ConfirmAction(self, _("Set up library directories?"),
+                   _("You don't have any music library set up. "
+                     "Would you like to do that now?")).run():
+                prefs = PreferencesWindow(self)
+                prefs.set_page("library")
+                prefs.show()
+
+    def __add_bookmark(self, librarian, player):
+        if player.song:
+            position = player.get_position() // 1000
+            bookmarks = player.song.bookmarks
+            new_mark = (position, _("Bookmark Name"))
+            if new_mark not in bookmarks:
+                bookmarks.append(new_mark)
+                player.song.bookmarks = bookmarks
+
+    def __edit_bookmarks(self, librarian, player):
+        if player.song:
+            window = EditBookmarks(self, librarian, player)
+            window.show()
+
     def __key_pressed(self, player, event):
         if not player.song:
             return
@@ -350,7 +500,8 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
             return True
 
     def __destroy(self, *args):
-        GLib.source_remove(self.__delayed_setup)
+        if self.__delayed_setup:
+            GLib.source_remove(self.__delayed_setup)
         # The tray icon plugin tries to unhide QL because it gets disabled
         # on Ql exit. The window should stay hidden after destroy.
         self.show = lambda: None
@@ -391,7 +542,7 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
         else:
             if dirs:
                 copool.add(
-                    self.__library.scan, dirs, self.__status.bar.progress,
+                    self.__library.scan, dirs,
                     cofuncid="library", funcid="library")
 
     def __songlist_drag_data_recv(self, view, *args):
@@ -436,6 +587,10 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
     def __create_menu(self, player, library):
         ag = Gtk.ActionGroup('QuodLibetWindowActions')
 
+        def logging_cb(*args):
+            window = LoggingWindow(self)
+            window.show()
+
         actions = [
             ('Music', None, _("_Music")),
             ('AddFolders', Gtk.STOCK_ADD, _('_Add a Folder...'),
@@ -458,6 +613,8 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
              "", None, self.__filter_menu_actions),
             ("TopRated", Gtk.STOCK_FIND, _("_Top 40"),
              "", None, self.__filter_menu_actions),
+            ("All", Gtk.STOCK_FIND, _("All _Songs"),
+             "", None, self.__filter_menu_actions),
 
             ("Control", None, _("_Control")),
             ("EditTags", Gtk.STOCK_PROPERTIES, _("Edit _Tags"), "", None,
@@ -471,15 +628,9 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
             ("View", None, _("_View")),
             ("Help", None, _("_Help")),
             ("OutputLog", Gtk.STOCK_EDIT, _("_Output Log"),
-             None, None, lambda *args: LoggingWindow(self)),
+             None, None, logging_cb),
             ]
 
-        if const.DEBUG:
-            def cause_error(*args):
-                raise Exception
-            actions.append(("DebugCauseError", Gtk.STOCK_DIALOG_ERROR,
-                            _("_Cause an Error"), None, None, cause_error))
-
         actions.append(("Previous", Gtk.STOCK_MEDIA_PREVIOUS, None,
                         "<control>comma", None, self.__previous_song))
 
@@ -491,6 +642,23 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
 
         ag.add_actions(actions)
 
+        act = Gtk.ToggleAction("StopAfter",
+                               _("Stop After This Song"), None, "")
+        ag.add_action_with_accel(act, "<shift>space")
+
+        # access point for the tray icon
+        self.stop_after = act
+
+        act = Gtk.Action("AddBookmark", _("Add Bookmark"), None, Gtk.STOCK_ADD)
+        act.connect_object('activate', self.__add_bookmark,
+                           library.librarian, player)
+        ag.add_action_with_accel(act, "<ctrl>D")
+
+        act = Gtk.Action("EditBookmarks", _(u"Edit Bookmarks…"), None, "")
+        act.connect_object('activate', self.__edit_bookmarks,
+                           library.librarian, player)
+        ag.add_action_with_accel(act, "<ctrl>B")
+
         act = Gtk.Action("About", None, None, Gtk.STOCK_ABOUT)
         act.connect_object('activate', self.__show_about, player)
         ag.add_action_with_accel(act, None)
@@ -507,10 +675,6 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
             "RefreshLibrary", _("Re_fresh Library"), None, Gtk.STOCK_REFRESH)
         act.connect('activate', self.__rebuild, False)
         ag.add_action_with_accel(act, None)
-        act = Gtk.Action(
-            "ReloadLibrary", _("Re_load Library"), None, Gtk.STOCK_REFRESH)
-        act.connect('activate', self.__rebuild, True)
-        ag.add_action_with_accel(act, None)
 
         for tag_, lab in [
             ("genre", _("Filter on _Genre")),
@@ -559,37 +723,35 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
                 continue
             action = "Browser" + Kind.__name__
             label = Kind.accelerated_name
-            act = Gtk.Action(action, label, None, None)
-            act.connect_object('activate', LibraryBrowser, Kind, library)
+            act = Gtk.Action.new(action, label, None, None)
+            act.connect_object('activate', LibraryBrowser.open, Kind, library)
             ag.add_action_with_accel(act, None)
 
         debug_menu = ""
         if const.DEBUG:
-            debug_menu = ("<separator/>"
-                          "<menuitem action='OutputLog'/>"
-                          "<menuitem action='DebugCauseError'/>")
+            debug_menu = (
+                "<separator/>"
+                "<menuitem action='OutputLog' always-show-image='true'/>")
 
-        self.ui = Gtk.UIManager()
-        self.ui.insert_action_group(ag, -1)
+        ui = Gtk.UIManager()
+        ui.insert_action_group(ag, -1)
         menustr = const.MENU % {"browsers": browsers.BrowseLibrary(),
                                 "views": browsers.ViewBrowser(),
                                 "debug": debug_menu}
-        self.ui.add_ui_from_string(menustr)
+        ui.add_ui_from_string(menustr)
 
         # Cute. So. UIManager lets you attach tooltips, but when they're
         # for menu items, they just get ignored. So here I get to actually
         # attach them.
-        self.ui.get_widget("/Menu/Music/RefreshLibrary").set_tooltip_text(
+        ui.get_widget("/Menu/Music/RefreshLibrary").set_tooltip_text(
                 _("Check for changes in your library"))
 
-        self.ui.get_widget("/Menu/Music/ReloadLibrary").set_tooltip_text(
-                _("Reload all songs in your library "
-                  "(this can take a long time)"))
-
-        self.ui.get_widget("/Menu/Filters/TopRated").set_tooltip_text(
+        ui.get_widget("/Menu/Filters/TopRated").set_tooltip_text(
                 _("The 40 songs you've played most (more than 40 may "
                   "be chosen if there are ties)"))
 
+        return ui
+
     def __show_about(self, player):
         about = AboutQuodLibet(self, player)
         about.run()
@@ -631,15 +793,14 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
 
         container = self.browser.__container = self.browser.pack(self.songpane)
 
-        # Look for a paned if the container is none
-        sub = container
-        if not isinstance(container, RPaned):
-            for child in container.get_children():
-                if isinstance(child, RPaned) and child is not self.songpane:
-                    sub = child
+        # find a paned and save the position
+        paned = None
+        for widget in qltk.find_widgets(container, RPaned):
+            if widget is not self.songpane:
+                paned = widget
+                break
 
-        # Save position if container is a RPaned
-        if isinstance(sub, RPaned):
+        if paned:
             try:
                 key = "%s_pos" % self.browser.__class__.__name__
                 val = config.getfloat("browsers", key)
@@ -647,10 +808,10 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
                 val = max(val, 0.1)
             except:
                 val = 0.4
-            sub.connect(
+            paned.connect(
                 'notify::position', self.__browser_configure, self.browser)
 
-            sub.set_relative(val)
+            paned.set_relative(val)
 
         player.replaygain_profiles[1] = self.browser.replaygain_profiles
         player.volume = player.volume
@@ -672,21 +833,22 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
         menu.set_label(text)
         menu.set_use_underline(True)
 
-    def __check_remove_song(self, player, song):
-        if song is None:
-            return
-        if not self.browser.dynamic(song):
-            iter = self.songlist.model.find(song)
-            if iter:
-                self.songlist.remove_iters([iter])
-
     def __song_ended(self, player, song, stopped):
-        self.__check_remove_song(player, song)
+        # check if the song should be removed base on the
+        # active filter of the current browser
+        active_filter = self.browser.active_filter
+        if song and active_filter and not active_filter(song):
+            iter_ = self.songlist.model.find(song)
+            if iter_:
+                self.songlist.remove_iters([iter_])
+
+        if self.stop_after.get_active():
+            player.paused = True
+            self.stop_after.set_active(False)
 
     def __song_changed(self, player, songs):
-        self.__update_title(player)
-        for song in songs:
-            self.__check_remove_song(player, song)
+        if player.info in songs:
+            self.__update_title(player)
 
     def __update_title(self, player):
         song = player.info
@@ -698,7 +860,8 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
     def __song_started(self, player, song):
         self.__update_title(player)
 
-        for wid in ["Jump", "Next", "EditTags", "Information"]:
+        for wid in ["Jump", "Next", "EditTags", "Information",
+                    "EditBookmarks", "AddBookmark", "StopAfter"]:
             self.ui.get_widget(
                 '/Menu/Control/' + wid).set_sensitive(bool(song))
         for wid in ["FilterAlbum", "FilterArtist", "FilterGenre"]:
@@ -817,16 +980,20 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
                 self.__make_query("#(playcount > %d)" % (songs[0] - 1))
             else:
                 self.__make_query("#(playcount > %d)" % (songs[-40] - 1))
+        elif name == "All":
+            self.browser.unfilter()
 
     def __rebuild(self, activator, force):
-        scan_libary(self.__library, force)
+        scan_library(self.__library, force)
 
     # Set up the preferences window.
     def __preferences(self, activator):
-        PreferencesWindow(self)
+        window = PreferencesWindow(self)
+        window.show()
 
     def __plugins(self, activator):
-        PluginWindow(self)
+        window = PluginWindow(self)
+        window.show()
 
     def open_location(self, action):
         name = GetStringDialog(self, _("Add a Location"),
@@ -848,52 +1015,55 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
                     self.__library.add([RemoteFile(name)])
 
     def open_chooser(self, action):
-        if not os.path.exists(self.last_dir):
-            self.last_dir = const.HOME
+        last_dir = self.last_dir
+        if not os.path.exists(last_dir):
+            last_dir = const.HOME
+
+        class MusicFolderChooser(FolderChooser):
+            def __init__(self, parent, init_dir):
+                super(MusicFolderChooser, self).__init__(
+                    parent, _("Add Music"), init_dir)
+
+                cb = Gtk.CheckButton(_("Watch this folder for new songs"))
+                # enable if no folders are being watched
+                cb.set_active(not get_scan_dirs())
+                cb.show()
+                self.set_extra_widget(cb)
+
+            def run(self):
+                fns = super(MusicFolderChooser, self).run()
+                cb = self.get_extra_widget()
+                return fns, cb.get_active()
+
+        class MusicFileChooser(FileChooser):
+            def __init__(self, parent, init_dir):
+                super(MusicFileChooser, self).__init__(
+                    parent, _("Add Music"), formats.filter, init_dir)
 
         if action.get_name() == "AddFolders":
-            chooser = FolderChooser(self, _("Add Music"), self.last_dir)
-            cb = Gtk.CheckButton(_("Watch this folder for new songs"))
-            cb.set_active(not config.get("settings", "scan"))
-            cb.show()
-            chooser.set_extra_widget(cb)
-        else:
-            chooser = FileChooser(
-                self, _("Add Music"), formats.filter, self.last_dir)
-            cb = None
-
-        fns = chooser.run()
-        chooser.destroy()
-        if fns:
-            if action.get_name() == "AddFolders":
+            dialog = MusicFolderChooser(self, last_dir)
+            fns, do_watch = dialog.run()
+            dialog.destroy()
+            if fns:
+                # scan them
                 self.last_dir = fns[0]
                 copool.add(self.__library.scan, fns, funcid="library")
-            else:
-                self.last_dir = os.path.basename(fns[0])
-                for filename in map(os.path.realpath, map(util.fsnative, fns)):
-                    if filename in self.__library:
-                        continue
-                    song = self.__library.add_filename(filename)
-                    if not song:
-                        from traceback import format_exception_only as feo
-                        tb = feo(sys.last_type, sys.last_value)
-                        msg = _("%s could not be added to your library.\n\n")
-                        msg %= util.escape(util.fsdecode(
-                            os.path.basename(filename)))
-                        msg += util.escape("".join(tb).decode(
-                            const.ENCODING, "replace"))
-                        d = ErrorMessage(self, _("Unable to add song"), msg)
-                        d.label.set_selectable(True)
-                        d.run()
-                        continue
 
-        if cb and cb.get_active():
-            dirs = util.split_scan_dirs(config.get("settings", "scan"))
-            for fn in fns:
-                if fn not in dirs:
-                    dirs.append(fn)
-            dirs = ":".join(dirs)
-            config.set("settings", "scan", dirs)
+                # add them as library scan directory
+                if do_watch:
+                    dirs = get_scan_dirs()
+                    for fn in fns:
+                        if fn not in dirs:
+                            dirs.append(fn)
+                    set_scan_dirs(dirs)
+        else:
+            dialog = MusicFileChooser(self, last_dir)
+            fns = dialog.run()
+            dialog.destroy()
+            if fns:
+                self.last_dir = os.path.dirname(fns[0])
+                for filename in map(os.path.realpath, map(util.fsnative, fns)):
+                    self.__library.add_filename(filename)
 
     def __songs_popup_menu(self, songlist):
         path, col = songlist.get_cursor()
@@ -906,12 +1076,16 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
     def __current_song_prop(self, *args):
         song = app.player.song
         if song:
-            SongProperties(self.__library.librarian, [song], parent=self)
+            librarian = self.__library.librarian
+            window = SongProperties(librarian, [song], parent=self)
+            window.show()
 
     def __current_song_info(self, *args):
         song = app.player.song
         if song:
-            Information(self.__library.librarian, [song], self)
+            librarian = self.__library.librarian
+            window = Information(librarian, [song], self)
+            window.show()
 
     def __hide_menus(self):
         menus = {'genre': ["/Menu/Filters/FilterGenre",
@@ -923,7 +1097,7 @@ class QuodLibetWindow(Gtk.Window, PersistentWindowMixin):
                  None: ["/Menu/Filters/PlayedRecently",
                         "/Menu/Filters/AddedRecently",
                         "/Menu/Filters/TopRated",
-                        "/Menu/Filters/TopRated"]}
+                        "/Menu/Filters/All"]}
         for key, widgets in menus.items():
             c = self.browser.can_filter(key)
             for widget in widgets:
