@@ -19,6 +19,8 @@ from quodlibet.qltk._editpane import EditPane, FilterCheckButton
 from quodlibet.qltk._editpane import EditingPluginHandler
 from quodlibet.qltk.views import TreeViewColumn
 from quodlibet.qltk.wlw import WritingWindow
+from quodlibet.util.path import fsdecode, fsencode
+from quodlibet.util.path import strip_win32_incompat_from_path
 
 
 class SpacesToUnderscores(FilterCheckButton):
@@ -46,7 +48,8 @@ class StripWindowsIncompat(FilterCheckButton):
             self.set_no_show_all(True)
 
     def filter(self, original, filename):
-        return util.strip_win32_incompat(filename)
+        assert isinstance(filename, unicode)
+        return fsdecode(strip_win32_incompat_from_path(fsencode(filename)))
 
 
 class StripDiacriticals(FilterCheckButton):
@@ -123,7 +126,8 @@ class RenameFiles(EditPane):
     def __rename(self, library):
         model = self.view.get_model()
         win = WritingWindow(self, len(model))
-        was_changed = []
+        win.show()
+        was_changed = set()
         skip_all = False
         self.view.freeze_child_notify()
 
@@ -146,8 +150,8 @@ class RenameFiles(EditPane):
                       "Possibly the target file already exists, "
                       "or you do not have permission to make the "
                       "new file or remove the old one.") % (
-                    util.escape(util.fsdecode(oldname)),
-                    util.escape(util.fsdecode(newname))),
+                    util.escape(fsdecode(oldname)),
+                    util.escape(fsdecode(newname))),
                     buttons=Gtk.ButtonsType.NONE)
                 msg.add_buttons(*buttons)
                 msg.set_default_response(Gtk.ResponseType.OK)
@@ -190,7 +194,7 @@ class RenameFiles(EditPane):
                 self.combo.write(const.NBP)
 
         orignames = [song["~filename"] for song in songs]
-        newnames = [util.fsdecode(util.fsencode(pattern.format(song)))
+        newnames = [fsdecode(fsencode(pattern.format(song)))
                     for song in songs]
         for f in self.filters:
             if f.active:
@@ -198,7 +202,7 @@ class RenameFiles(EditPane):
 
         model.clear()
         for song, newname in zip(songs, newnames):
-            basename = util.fsdecode(song("~basename"))
+            basename = fsdecode(song("~basename"))
             model.append(row=[song, basename, newname])
         self.preview.set_sensitive(False)
         self.save.set_sensitive(bool(self.combo.get_child().get_text()))
