@@ -1,5 +1,6 @@
 # Copyright 2005 Joe Wreschnig
 #           2012 Christoph Reiter
+#      2011-2013 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -23,6 +24,7 @@ from quodlibet.qltk.views import AllTreeView, DragScroll
 from quodlibet.qltk.ratingsmenu import RatingsMenuItem
 from quodlibet.qltk.ratingsmenu import ConfirmRateMultipleDialog
 from quodlibet.qltk.songmodel import PlaylistModel
+from quodlibet.util.path import fsdecode, unexpand
 from quodlibet.util.uri import URI
 from quodlibet.formats._audio import TAG_TO_SORT, FILESYSTEM_TAGS, AudioFile
 from quodlibet.qltk.sortdialog import SortDialog
@@ -129,7 +131,7 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
             return True
 
         def _cdf(self, column, cell, model, iter, tag):
-            text = model[iter][0].comma(tag)
+            text = model.get_value(iter).comma(tag)
             if not self._needs_update(text):
                 return
             cell.set_property('text', text)
@@ -181,7 +183,7 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
     class DateColumn(TextColumn):
         # The '~#' keys that are dates.
         def _cdf(self, column, cell, model, iter, tag):
-            stamp = model[iter][0](tag)
+            stamp = model.get_value(iter)(tag)
             if not self._needs_update(stamp):
                 return
             if not stamp:
@@ -215,7 +217,8 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         # Render ~#rating directly (simplifies filtering, saves
         # a function call).
         def _cdf(self, column, cell, model, iter, tag):
-                value = model[iter][0].get("~#rating", const.DEFAULT_RATING)
+                value = model.get_value(iter).get(
+                    "~#rating", config.RATINGS.default)
                 if not self._needs_update(value):
                     return
                 cell.set_property('text', util.format_rating(value))
@@ -232,7 +235,7 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         # Optimize for non-synthesized keys by grabbing them directly.
         # Used for any tag without a '~' except 'title'.
         def _cdf(self, column, cell, model, iter, tag):
-            value = model[iter][0].get(tag, "")
+            value = model.get_value(iter).get(tag, "")
             if not self._needs_update(value):
                 return
             cell.set_property('text', value.replace("\n", ", "))
@@ -241,15 +244,15 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         # Contains text in the filesystem encoding, so needs to be
         # decoded safely (and also more slowly).
         def _cdf(self, column, cell, model, iter, tag):
-            value = model[iter][0].comma(tag)
+            value = model.get_value(iter).comma(tag)
             if not self._needs_update(value):
                 return
-            cell.set_property('text', util.unexpand(util.fsdecode(value)))
+            cell.set_property('text', unexpand(fsdecode(value)))
 
     class NumericColumn(TextColumn):
         # Any '~#' keys except dates.
         def _cdf(self, column, cell, model, iter, tag):
-            value = model[iter][0].comma(tag)
+            value = model.get_value(iter).comma(tag)
             if not self._needs_update(value):
                 return
             text = unicode(value)
@@ -263,7 +266,7 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
 
     class LengthColumn(NumericColumn):
         def _cdf(self, column, cell, model, iter, tag):
-            value = model[iter][0].get("~#length", 0)
+            value = model.get_value(iter).get("~#length", 0)
             if not self._needs_update(value):
                 return
             text = util.format_time(value)
@@ -275,7 +278,7 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
 
     class FilesizeColumn(NumericColumn):
         def _cdf(self, column, cell, model, iter, tag):
-            value = model[iter][0].get("~#filesize", 0)
+            value = model.get_value(iter).get("~#filesize", 0)
             if not self._needs_update(value):
                 return
             text = util.format_size(value)
@@ -380,25 +383,18 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
 
         self.set_search_equal_func(self.__search_func, None)
 
-        self.accelerators = Gtk.AccelGroup()
-        key, mod = Gtk.accelerator_parse("<alt>Return")
-        self.accelerators.connect(
-            key, mod, 0, lambda *args: self.__song_properties(librarian))
-        key, mod = Gtk.accelerator_parse("<control>I")
-        self.accelerators.connect(
-            key, mod, 0, lambda *args: self.__information(librarian))
-
         self.connect('destroy', self.__destroy)
 
     def __destroy(self, *args):
         self.info.destroy()
         self.handler_block(self.__csig)
-        map(self.remove_column, self.get_columns())
+        for column in self.get_columns():
+            self.remove_column(column)
         self.handler_unblock(self.__csig)
 
     def __search_func(self, model, column, key, iter, *args):
         for column in self.get_columns():
-            value = model.get_value(iter, 0)(column.header_name)
+            value = model.get_value(iter)(column.header_name)
             if not isinstance(value, basestring):
                 continue
             elif key in value.lower() or key in value:
@@ -460,9 +456,8 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
                       "song lists or the queue.")).run()
                 Gdk.drag_abort(ctx, etime)
                 return
-            filenames = [song("~filename") for song in songs]
-            type_ = Gdk.atom_intern("text/x-quodlibet-songs", True)
-            sel.set(type_, 8, "\x00".join(filenames))
+
+            qltk.selection_set_songs(sel, songs)
             if ctx.get_actions() & Gdk.DragAction.MOVE:
                 self.__drag_iters = map(model.get_iter, paths)
             else:
@@ -482,7 +477,7 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
     def __drag_data_received(self, view, ctx, x, y, sel, info, etime, library):
         model = view.get_model()
         if info == DND_QL:
-            filenames = sel.get_data().split("\x00")
+            filenames = qltk.selection_get_filenames(sel)
             move = (Gtk.drag_get_source_widget(ctx) == view)
         elif info == DND_URI_LIST:
             def to_filename(s):
@@ -588,7 +583,7 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         sd.hide()
 
     def __button_press(self, view, event, librarian):
-        if event.button != 1:
+        if event.button != Gdk.BUTTON_PRIMARY:
             return
         x, y = map(int, [event.x, event.y])
         try:
@@ -604,23 +599,23 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
             song = view.get_model()[path][0]
             l = Gtk.Label()
             l.show()
-            l.set_text(util.format_rating(util.RATING_PRECISION, blank=False))
+            l.set_text(config.RATINGS.full_symbol)
             width = l.get_preferred_size()[1].width
             l.destroy()
             if not width:
                 return False
-
+            precision = config.RATINGS.precision
             count = int(float(cellx - 5) / width) + 1
-            rating = max(0.0, min(1.0, count * util.RATING_PRECISION))
-            if (rating <= util.RATING_PRECISION and
-                    song("~#rating") == util.RATING_PRECISION):
+            rating = max(0.0, min(1.0, count * precision))
+            if (rating <= precision and
+                    song("~#rating") == precision):
                 rating = 0
             self.__set_rating(rating, [song], librarian)
 
     def __set_rating(self, value, songs, librarian):
         count = len(songs)
         if (count > 1 and
-            config.getboolean("browsers", "rating_confirm_multiple")):
+                config.getboolean("browsers", "rating_confirm_multiple")):
             dialog = ConfirmRateMultipleDialog(self, count, value)
             if dialog.run() != Gtk.ResponseType.YES:
                 return
@@ -630,7 +625,7 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
 
     def __key_press(self, songlist, event, librarian):
         if event.string in ['0', '1', '2', '3', '4']:
-            rating = min(1.0, int(event.string) * util.RATING_PRECISION)
+            rating = min(1.0, int(event.string) * config.RATINGS.precision)
             self.__set_rating(rating, self.get_selected_songs(), librarian)
             return True
         elif qltk.is_accel(event, "<ctrl>Return") or \
@@ -640,6 +635,18 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         elif qltk.is_accel(event, "<control>F"):
             self.emit('start-interactive-search')
             return True
+        elif qltk.is_accel(event, "<alt>Return"):
+            songs = self.get_selected_songs()
+            if songs:
+                window = SongProperties(librarian, songs, parent=self)
+                window.show()
+            return True
+        elif qltk.is_accel(event, "<control>I"):
+            songs = self.get_selected_songs()
+            if songs:
+                window = Information(librarian, songs, self)
+                window.show()
+            return True
         return False
 
     def __enqueue(self, songs):
@@ -833,13 +840,8 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         else:
             self.set_sort_by(None, refresh=False)
 
-        # Doing set_model(None) resets the sort indicator, so we need to
-        # remember it before doing that.
-        sorts = map(Gtk.TreeViewColumn.get_sort_indicator, self.get_columns())
-        self.set_model(None)
-        model.set(songs)
-        self.set_model(model)
-        map(Gtk.TreeViewColumn.set_sort_indicator, self.get_columns(), sorts)
+        with self.without_model() as model:
+            model.set(songs)
 
         # the song selection has queued a change now, cancel that and
         # pass the songs manually
@@ -849,18 +851,16 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         songs = []
 
         def func(model, path, iter_, user_data):
-            songs.append(model.get_value(iter_, 0))
+            songs.append(model.get_value(iter_))
         selection = self.get_selection()
         selection.selected_foreach(func, None)
         return songs
 
     def __song_updated(self, librarian, songs):
         """Only update rows that are currently displayed.
-        Warning: This makes the row-changed signal useless."""
-        #pygtk 2.12: prevent invalid ranges or GTK asserts
-        if not self.get_realized() or \
-                self.get_path_at_pos(0, 0) is None:
-            return
+        Warning: This makes the row-changed signal useless.
+        """
+
         vrange = self.get_visible_range()
         if vrange is None:
             return
@@ -908,7 +908,8 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
                 songs = [app.player.song]
             else:
                 return
-        SongProperties(librarian, songs, parent=self)
+        window = SongProperties(librarian, songs, parent=self)
+        window.show()
 
     def __information(self, librarian):
         model, rows = self.get_selection().get_selected_rows()
@@ -920,7 +921,8 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
                 songs = [app.player.song]
             else:
                 return
-        Information(librarian, songs, self)
+        window = Information(librarian, songs, self)
+        window.show()
 
     # Build a new filter around our list model, set the headers to their
     # new values.
@@ -931,7 +933,8 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         self.handler_block(self.__csig)
 
         old_sort = self.is_sorted() and self.get_sort_by()
-        map(self.remove_column, self.get_columns())
+        for column in self.get_columns():
+            self.remove_column(column)
 
         if self.CurrentColumn is not None:
             self.append_column(self.CurrentColumn())
@@ -1066,12 +1069,15 @@ class SongList(AllTreeView, DragScroll, util.InstanceTracker):
         # Prefs has to import SongList, so do this here to avoid
         # a circular import.
         from quodlibet.qltk.prefs import PreferencesWindow
-        PreferencesWindow(self).set_page("songlist")
+        window = PreferencesWindow(self)
+        window.show()
+        window.set_page("songlist")
 
     def __showmenu(self, column, event=None):
-        time = Gtk.get_current_event_time()
-        if event is not None and event.button != 3:
-            return
+        time = event.time if event else Gtk.get_current_event_time()
+
+        if event is not None and event.button != Gdk.BUTTON_SECONDARY:
+            return False
 
         if event:
             self.__getmenu(column).popup(None, None, None, None,
