@@ -18,8 +18,6 @@ from quodlibet.qltk.window import Window, UniqueWindow
 class ScrolledWindow(Gtk.ScrolledWindow):
     """Draws a border around all edges that don't touch the parent window"""
 
-    __gsignals__ = {'size-allocate': 'override'}
-
     def do_size_allocate(self, alloc):
         if self.get_shadow_type() == Gtk.ShadowType.NONE:
             return Gtk.ScrolledWindow.do_size_allocate(self, alloc)
@@ -43,6 +41,28 @@ class ScrolledWindow(Gtk.ScrolledWindow):
         # https://bugzilla.gnome.org/show_bug.cgi?id=694844
         border.left = border.top = border.right = border.bottom = 1
 
+        # XXX: ugly, ugly hack
+        # Pretend the main window toolbar is the top of the window.
+        # This removes the top border in case the ScrolledWindow
+        # is drawn right below the toolbar.
+        try:
+            top_bar = toplevel.top_bar
+            if not isinstance(top_bar, Gtk.Widget):
+                raise TypeError
+        except (AttributeError, TypeError):
+            # In case the window border is at the top, we expect the menubar
+            # there, so draw the normal border
+            border.top = 0
+        else:
+            top_alloc = top_bar.get_allocation()
+            top_ctx = top_bar.get_style_context()
+            b = top_ctx.get_border(top_bar.get_state_flags())
+            # only if the toolbar has a border we hide our own.
+            # seems to work, even tough it doesn't for getting the
+            # Notebook/ScrolledWindow border :/
+            if b.bottom:
+                dy -= top_alloc.y + top_alloc.height
+
         # Don't remove the border if the border is drawn inside
         # and the scrollbar on that edge is visible
         bottom = left = right = top = False
@@ -103,8 +123,6 @@ class Notebook(Gtk.Notebook):
     label is given, the page's 'title' attribute (either a string or
     a widget) is used."""
 
-    __gsignals__ = {'size-allocate': 'override'}
-
     def do_size_allocate(self, alloc):
         ctx = self.get_style_context()
         border = ctx.get_border(self.get_state_flags())
@@ -122,6 +140,9 @@ class Notebook(Gtk.Notebook):
         dx = x2 - x1
         dy = y2 - y1
 
+        # all 0 since gtk+ 3.12..
+        border.left = border.top = border.right = border.bottom = 1
+
         width, height = toplevel.get_size()
         if alloc.y + alloc.height + dy == height:
             alloc.height += border.bottom
@@ -164,8 +185,9 @@ def Frame(label, child=None):
     return frame
 
 
-def Alignment(child=None, top=0, bottom=0, left=0, right=0, border=0):
-    align = Gtk.Alignment(xscale=1.0, yscale=1.0)
+def Alignment(child=None, top=0, bottom=0, left=0, right=0, border=0,
+              **kwargs):
+    align = Gtk.Alignment(**kwargs)
     align.set_padding(top + border, bottom + border,
                       left + border, right + border)
     if child:
@@ -175,21 +197,34 @@ def Alignment(child=None, top=0, bottom=0, left=0, right=0, border=0):
 
 def MenuItem(label, stock_id):
     """An ImageMenuItem with a custom label and stock image."""
+
     item = Gtk.ImageMenuItem.new_with_mnemonic(label)
-    item.set_image(Gtk.Image(stock=stock_id, icon_size=Gtk.IconSize.MENU))
+    item.set_always_show_image(True)
+    if Gtk.stock_lookup(stock_id):
+        image = Gtk.Image.new_from_stock(stock_id, Gtk.IconSize.MENU)
+    else:
+        image = Gtk.Image.new_from_icon_name(stock_id, Gtk.IconSize.MENU)
+    image.show()
+    item.set_image(image)
     return item
 
 
 def Button(label, stock_id, size=Gtk.IconSize.BUTTON):
     """A Button with a custom label and stock image. It should pack
     exactly like a stock button."""
+
     align = Gtk.Alignment(xscale=0.0, yscale=1.0, xalign=0.5, yalign=0.5)
     hbox = Gtk.HBox(spacing=2)
-    hbox.pack_start(Gtk.Image.new_from_stock(stock_id, size), True, True, 0)
-    label = Gtk.Label(label)
+    if Gtk.stock_lookup(stock_id):
+        image = Gtk.Image.new_from_stock(stock_id, size)
+    else:
+        image = Gtk.Image.new_from_icon_name(stock_id, size)
+    hbox.pack_start(image, True, True, 0)
+    label = Gtk.Label(label=label)
     label.set_use_underline(True)
     hbox.pack_start(label, True, True, 0)
     align.add(hbox)
+    align.show_all()
     button = Gtk.Button()
     button.add(align)
     return button
@@ -268,7 +303,7 @@ class ConfigRPaned(RPaned):
 
 
 class ConfigRHPaned(ConfigRPaned):
-    ORIENTATION = Gtk.Orientation.VERTICAL
+    ORIENTATION = Gtk.Orientation.HORIZONTAL
 
 
 class ConfigRVPaned(ConfigRPaned):
