@@ -6,13 +6,16 @@
 
 import os
 import shelve
-from tempfile import mkstemp
+
+from tests import mkstemp
 
 from quodlibet.plugins.events import EventPlugin
 
+
 class PickleMe(object):
     pass
 
+
 class PickleTestPlugin(EventPlugin):
     PLUGIN_ID = "pickle_test"
     PLUGIN_NAME = "This is a test"
