@@ -1,11 +1,12 @@
-from tests import TestCase, add
+from tests import TestCase
 
 from gi.repository import Gtk
 
 import quodlibet.browsers.search
 import quodlibet.config
 
-from quodlibet.browsers.search import EmptyBar, SearchBar
+from quodlibet.browsers.search import SearchBar
+from quodlibet.browsers.empty import EmptyBar
 from quodlibet.formats._audio import AudioFile
 from quodlibet.library import SongLibrary, SongLibrarian
 
@@ -35,8 +36,10 @@ SONGS = [AudioFile({
                 "labelid": "12345-6",
                 "~filename": "/dev/sh"})]
 
+
 class TEmptyBar(TestCase):
     Bar = EmptyBar
+
     def setUp(self):
         quodlibet.config.init()
         quodlibet.browsers.search.library = SongLibrary()
@@ -53,7 +56,8 @@ class TEmptyBar(TestCase):
         self.expected = None
 
     def _do(self):
-        while Gtk.events_pending(): Gtk.main_iteration()
+        while Gtk.events_pending():
+            Gtk.main_iteration()
         self.failUnless(self.expected is None)
 
     def test_can_filter(self):
@@ -111,8 +115,7 @@ class TEmptyBar(TestCase):
         self.bar.destroy()
         quodlibet.browsers.search.library.destroy()
         quodlibet.config.quit()
-add(TEmptyBar)
+
 
 class TSearchBar(TEmptyBar):
     Bar = SearchBar
-add(TSearchBar)
