@@ -1,30 +1,110 @@
 # -*- coding: utf-8 -*-
 
-from tests import add, TestCase
+from tests import TestCase, DATA_DIR, mkstemp
 
 import os
 import shutil
-import tempfile
+import StringIO
 
 from quodlibet import config, const
+from quodlibet.formats._image import EmbeddedImage
 from quodlibet.formats.mp3 import MP3File
 from quodlibet.formats._id3 import ID3hack
 
 import mutagen
 
+
+class TID3Images(TestCase):
+
+    def setUp(self):
+        config.init()
+        fd, self.filename = mkstemp(".mp3")
+        os.close(fd)
+        shutil.copy(os.path.join(DATA_DIR, 'silence-44-s.mp3'), self.filename)
+
+    def tearDown(self):
+        os.remove(self.filename)
+        config.quit()
+
+    def test_can_change_images(self):
+        self.failUnless(MP3File(self.filename).can_change_images)
+
+    def test_get_primary_image(self):
+        self.failIf(MP3File(self.filename).has_images)
+
+        f = mutagen.File(self.filename)
+        apic = mutagen.id3.APIC(encoding=3, mime="image/jpeg", type=4,
+                                desc="foo", data="bar")
+        f.tags.add(apic)
+        f.save()
+
+        song = MP3File(self.filename)
+        self.failUnless(song.has_images)
+        image = song.get_primary_image()
+        self.assertEqual(image.mime_type, "image/jpeg")
+        fn = image.file
+        self.failUnlessEqual(fn.read(), "bar")
+
+        apic = mutagen.id3.APIC(encoding=3, mime="image/jpeg", type=3,
+                                desc="xx", data="bar2")
+        f.tags.add(apic)
+        f.save()
+
+        song = MP3File(self.filename)
+        self.failUnless(song.has_images)
+        image = song.get_primary_image()
+        self.failUnlessEqual(image.file.read(), "bar2")
+
+        # get_images()
+        images = song.get_images()
+        self.assertTrue(images and len(images) == 2)
+        self.assertEqual(images[0].type, 3)
+        self.assertEqual(images[1].type, 4)
+
+    def test_clear_images(self):
+        f = mutagen.File(self.filename)
+        apic = mutagen.id3.APIC(encoding=3, mime="image/jpeg", type=4,
+                                desc="foo", data="bar")
+        f.tags.add(apic)
+        f.save()
+
+        song = MP3File(self.filename)
+        self.failUnless(song.has_images)
+        song.clear_images()
+
+        song = MP3File(self.filename)
+        self.assertFalse(song.has_images)
+
+    def test_set_image(self):
+        fileobj = StringIO.StringIO("foo")
+        image = EmbeddedImage(fileobj, "image/jpeg", 10, 10, 8)
+
+        song = MP3File(self.filename)
+        self.failIf(song.has_images)
+        song.set_image(image)
+        self.assertTrue(song.has_images)
+
+        song = MP3File(self.filename)
+        self.assertTrue(song.has_images)
+        self.assertEqual(song.get_primary_image().mime_type, "image/jpeg")
+
+
 class TID3File(TestCase):
     def setUp(self):
         config.init()
-        self.filename = tempfile.mkstemp(".mp3")[1]
-        shutil.copy(os.path.join('tests', 'data', 'silence-44-s.mp3'), self.filename)
-        self.filename2 = tempfile.mkstemp(".mp3")[1]
-        shutil.copy(os.path.join('tests', 'data', 'mutagen-bug.mp3'), self.filename2)
+        fd, self.filename = mkstemp(".mp3")
+        os.close(fd)
+        shutil.copy(os.path.join(DATA_DIR, 'silence-44-s.mp3'), self.filename)
+        fd, self.filename2 = mkstemp(".mp3")
+        os.close(fd)
+        shutil.copy(os.path.join(DATA_DIR, 'mutagen-bug.mp3'), self.filename2)
 
     def test_optional_POPM_count(self):
         #http://code.google.com/p/quodlibet/issues/detail?id=364
         f = mutagen.File(self.filename)
         f.tags.add(mutagen.id3.POPM(const.EMAIL, 42))
-        try: f.save()
+        try:
+            f.save()
         except TypeError:
             #http://code.google.com/p/mutagen/issues/detail?id=33
             pass
@@ -51,8 +131,9 @@ class TID3File(TestCase):
         f = mutagen.File(self.filename)
         try:
             lang = u'free-text'
-            f.tags.add(mutagen.id3.TXXX(encoding=3, desc=u'QuodLibet::language',
-                                        text=lang))
+            f.tags.add(
+                mutagen.id3.TXXX(encoding=3, desc=u'QuodLibet::language',
+                                 text=lang))
             f.save()
             self.assertEquals(MP3File(self.filename)['language'], lang)
         finally:
@@ -82,9 +163,11 @@ class TID3File(TestCase):
         finally:
             f.delete()
 
-
     def test_write_lang_freetext(self):
-        """Tests that if you don't use an ISO 639-2 code, TXXX gets populated"""
+        """Tests that if you don't use an ISO 639-2 code,
+        TXXX gets populated
+        """
+
         af = MP3File(self.filename)
         for val in ["free-text", "foo", "de", "en"]:
             af["language"] = val
@@ -140,30 +223,6 @@ class TID3File(TestCase):
         f.save()
         self.failUnless(MP3File(self.filename)("~#length") > 0)
 
-    def test_load_apic(self):
-        self.failIf(MP3File(self.filename)("~picture"))
-
-        f = mutagen.File(self.filename)
-        apic = mutagen.id3.APIC(encoding=3, mime="image/jpeg", type=4,
-                                desc="foo", data="bar")
-        f.tags.add(apic)
-        f.save()
-
-        song = MP3File(self.filename)
-        self.failUnless(song("~picture"))
-        fn = song.get_format_cover()
-        self.failUnlessEqual(fn.read(), "bar")
-
-        apic = mutagen.id3.APIC(encoding=3, mime="image/jpeg", type=3,
-                                desc="xx", data="bar2")
-        f.tags.add(apic)
-        f.save()
-
-        song = MP3File(self.filename)
-        self.failUnless(song("~picture"))
-        fn = song.get_format_cover()
-        self.failUnlessEqual(fn.read(), "bar2")
-
     def test_load_tcon(self):
         # check if the mutagen preprocessing is used
         f = mutagen.File(self.filename)
@@ -484,7 +543,7 @@ class TID3File(TestCase):
         f.delete()
         f.save()
         song = MP3File(self.filename)
-        song.get_format_cover()
+        song.get_primary_image()
         song.write()
 
     def test_distrust_latin1(self):
@@ -511,9 +570,13 @@ class TID3File(TestCase):
 
         self.failUnlessEqual(MP3File(self.filename)("artist"), x)
 
+    def test_handled_txxx_encoding(self):
+        song = MP3File(self.filename)
+        song['albumartistsort'] = u'Dvo\u0159\xe1k, Anton\xedn'
+        song["replaygain_track_peak"] = u'Dvo\u0159\xe1k, Anton\xedn'
+        song.write()
+
     def tearDown(self):
         os.unlink(self.filename2)
         os.unlink(self.filename)
         config.quit()
-
-add(TID3File)
