@@ -1,4 +1,4 @@
-from tests import TestCase, add
+from tests import TestCase, mkstemp, AbstractTestCase
 
 import os
 
@@ -6,9 +6,9 @@ from quodlibet import formats
 from quodlibet import config
 
 from shutil import copyfileobj
-from tempfile import mkstemp
 
-class TestMetaData(TestCase):
+
+class TestMetaData(AbstractTestCase):
     base = 'tests/data/silence-44-s'
 
     def setUp(self):
@@ -64,7 +64,7 @@ class TestMetaData(TestCase):
 
     def test_genre(self): # unusual special handling for mp3s
         self._test_tag('genre', [u'Pop', u'Rock\nClassical', u'Big Bird',
-             u'\u30a2\u30cb\u30e1\u30b5\u30f3\u30c8\u30e9',])
+             u'\u30a2\u30cb\u30e1\u30b5\u30f3\u30c8\u30e9'])
 
     def test_odd_performer(self):
         values = [u"A Person", u"Another"]
@@ -86,12 +86,18 @@ for ext in formats._infos.keys():
 
         extra_tests = {}
         for tag in tags:
-            if tag in ['artist', 'date', 'genre']: continue
-            def test_tag(self, tag=tag): self._test_tag(tag, [u'a'])
+            if tag in ['artist', 'date', 'genre']:
+                continue
+
+            def test_tag(self, tag=tag):
+                self._test_tag(tag, [u'a'])
             extra_tests['test_tag_' + tag] = test_tag
-            def test_tags(self, tag=tag): self._test_tag(tag, [u'b\nc'])
+
+            def test_tags(self, tag=tag):
+                self._test_tag(tag, [u'b\nc'])
             extra_tests['test_tags_' + tag] = test_tags
 
-        testcase = type('MetaData' + ext, (TestMetaData,), extra_tests)
+        name = 'MetaData' + ext
+        testcase = type(name, (TestMetaData,), extra_tests)
         testcase.ext = ext
-        add(testcase)
+        globals()[name] = testcase
