@@ -1,4 +1,4 @@
-from tests import TestCase, add
+from tests import TestCase, mkstemp
 
 import os
 
@@ -6,6 +6,7 @@ from quodlibet import config
 from quodlibet.formats._audio import AudioFile
 from quodlibet.util.songwrapper import SongWrapper, ListWrapper
 
+
 class TSongWrapper(TestCase):
 
     psong = AudioFile({
@@ -13,19 +14,23 @@ class TSongWrapper(TestCase):
         "title": "more songs",
         "discnumber": "2/2", "tracknumber": "1",
         "artist": "Foo\nI have two artists", "album": "Bar",
-        "~bookmark": "2:10 A bookmark" })
+        "~bookmark": "2:10 A bookmark"})
     pwrap = SongWrapper(psong)
 
     def setUp(self):
+        fd, self.filename = mkstemp()
+        os.close(fd)
         config.init()
         self.wrap = SongWrapper(AudioFile(
-            {"title": "woo", "~filename": "/dev/null"}))
+            {"title": "woo", "~filename": self.filename}))
 
     def tearDown(self):
+        os.unlink(self.filename)
         config.quit()
 
     def test_slots(self):
-        def breakme(): self.wrap.woo = 1
+        def breakme():
+            self.wrap.woo = 1
         self.failUnlessRaises(AttributeError, breakme)
 
     def test_cmp(self):
@@ -60,7 +65,7 @@ class TSongWrapper(TestCase):
         self.failUnless(self.wrap._needs_write)
 
     def test_realkeys(self):
-        self.failUnlessEqual( self.pwrap.realkeys(), self.psong.realkeys())
+        self.failUnlessEqual(self.pwrap.realkeys(), self.psong.realkeys())
 
     def test_website(self):
         self.failUnlessEqual(self.pwrap.website(), self.psong.website())
@@ -89,7 +94,7 @@ class TSongWrapper(TestCase):
     def test_mtime(self):
         self.wrap._song.sanitize()
         self.failUnless(self.wrap.valid())
-        self.wrap["~#mtime"] = os.path.getmtime("/dev/null") - 2
+        self.wrap["~#mtime"] = os.path.getmtime(self.filename) - 2
         self.wrap._updated = False
         self.failIf(self.wrap.valid())
 
@@ -117,7 +122,6 @@ class TSongWrapper(TestCase):
         self.failUnlessEqual(self.psong["~bookmark"], "0:43 another mark")
         self.failUnlessEqual(self.psong.bookmarks, self.pwrap.bookmarks)
 
-add(TSongWrapper)
 
 class TListWrapper(TestCase):
     def test_empty(self):
@@ -133,6 +137,3 @@ class TListWrapper(TestCase):
         wrapped = ListWrapper([None, None])
         self.failUnless(len(wrapped) == 2)
         self.failUnlessEqual(wrapped, [None, None])
-add(TListWrapper)
-
-
