@@ -1,15 +1,31 @@
-from tests import TestCase, add
+from tests import TestCase
 
 from quodlibet.qltk._editpane import FilterCheckButton
 
+
 class FCB(FilterCheckButton):
     _section = _key = _label = "foo"
 
-class FCB2(FCB): _order = 1.0
-class FCB3(FCB): _order = 1.2
-class FCB4(FCB): _order = 1.3
-class FCB5(FCB): _order = 1.3
-class FCB1(FCB): _order = 1.4
+
+class FCB2(FCB):
+    _order = 1.0
+
+
+class FCB3(FCB):
+    _order = 1.2
+
+
+class FCB4(FCB):
+    _order = 1.3
+
+
+class FCB5(FCB):
+    _order = 1.3
+
+
+class FCB1(FCB):
+    _order = 1.4
+
 
 class TFilterCheckButton(TestCase):
     def setUp(self):
@@ -35,4 +51,3 @@ class TFilterCheckButton(TestCase):
     def tearDown(self):
         for cb in [self.fcb1, self.fcb2, self.fcb3, self.fcb4, self.fcb5]:
             cb.destroy()
-add(TFilterCheckButton)
