@@ -1,10 +1,14 @@
-from tests import TestCase, add
+from tests import TestCase
+
+from gi.repository import Gtk
 
 from quodlibet.qltk.chooser import FolderChooser, FileChooser
 import quodlibet.config
 
+
 class TFolderChooser(TestCase):
     Kind = FolderChooser
+
     def setUp(self):
         quodlibet.config.init()
 
@@ -13,17 +17,23 @@ class TFolderChooser(TestCase):
 
     def test_init_nodir(self):
         f = self.Kind(None, "A file chooser")
+        while Gtk.events_pending():
+            Gtk.main_iteration()
         f.destroy()
 
     def test_init_dir(self):
         f = self.Kind(None, "A file chooser", initial_dir="/home")
+        while Gtk.events_pending():
+            Gtk.main_iteration()
         f.destroy()
-add(TFolderChooser)
+
 
 class TFileChooser(TFolderChooser):
     Kind = FileChooser
 
     def test_filter(self):
         f = lambda *x: None
-        FileChooser(None, "foo", filter=f)
-add(TFileChooser)
+        x = FileChooser(None, "foo", filter=f)
+        while Gtk.events_pending():
+            Gtk.main_iteration()
+        x.destroy()
