@@ -6,7 +6,7 @@
 
 import sys
 
-from tests import TestCase, add
+from tests import TestCase
 
 from quodlibet.qltk.debugwindow import ExceptionDialog
 
@@ -19,9 +19,7 @@ class TExceptionDialog(TestCase):
             try:
                 raise Exception
             except Exception:
-                ExceptionDialog.excepthook(*sys.exc_info())
+                ExceptionDialog.from_except(*sys.exc_info())
                 ExceptionDialog.instance.destroy()
         finally:
             sys.stderr = old
-
-add(TExceptionDialog)
