@@ -4,7 +4,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from tests import TestCase, add
+from tests import TestCase
 
 from quodlibet.qltk.logging import LoggingWindow
 
@@ -13,5 +13,3 @@ class TLoggingWindow(TestCase):
     def test_window(self):
         w = LoggingWindow()
         w.destroy()
-
-add(TLoggingWindow)
