@@ -1,4 +1,4 @@
-from tests import TestCase, add
+from tests import TestCase
 
 from gi.repository import Gtk
 
@@ -7,6 +7,7 @@ from quodlibet.qltk.queue import QueueExpander
 from quodlibet.library import SongLibrary
 import quodlibet.config
 
+
 class TQueueExpander(TestCase):
     def setUp(self):
         quodlibet.config.init()
@@ -19,4 +20,3 @@ class TQueueExpander(TestCase):
     def tearDown(self):
         self.queue.destroy()
         quodlibet.config.quit()
-add(TQueueExpander)
