@@ -1,4 +1,4 @@
-from tests import TestCase, add
+from tests import TestCase
 
 from quodlibet.library import SongLibrary
 from quodlibet.qltk.songlist import SongList
@@ -8,6 +8,7 @@ import quodlibet.config
 class TSongList(TestCase):
     HEADERS = ["acolumn", "~#lastplayed", "~foo~bar", "~#rating",
                "~#length", "~dirname", "~#track"]
+
     def setUp(self):
         quodlibet.config.init()
         self.songlist = SongList(SongLibrary())
@@ -36,7 +37,10 @@ class TSongList(TestCase):
         self.songlist.set_sort_by(self.songlist.get_columns()[-1], tag="three")
         self.failUnlessEqual(self.songlist.get_sort_by(), ("three", True))
 
+    def test_inline_search_state(self):
+        self.assertEqual(self.songlist.get_search_column(), 0)
+        self.assertTrue(self.songlist.get_enable_search())
+
     def tearDown(self):
         self.songlist.destroy()
         quodlibet.config.quit()
-add(TSongList)
