@@ -1,9 +1,8 @@
-from tests import TestCase, add
+from tests import TestCase
 
 from quodlibet.qltk.sortdialog import SortDialog
 
+
 class TSortDialog(TestCase):
     def test(self):
         SortDialog(None).destroy()
-
-add(TSortDialog)
