@@ -1,8 +1,9 @@
-from tests import TestCase, add
+from tests import TestCase, AbstractTestCase
 
 from quodlibet.formats import USEFUL_TAGS
 
-class TagsCombo(TestCase):
+
+class TagsCombo(AbstractTestCase):
     def setUp(self):
         self.all = self.Kind()
         self.some = self.Kind(["artist", "album", "~people", "foobar"])
@@ -25,9 +26,10 @@ class TagsCombo(TestCase):
         self.all.destroy()
         self.some.destroy()
 
+
 class TTagsComboBox(TagsCombo):
     from quodlibet.qltk.tagscombobox import TagsComboBox as Kind
-add(TTagsComboBox)
+
 
 class TTagsComboBoxEntry(TagsCombo):
     from quodlibet.qltk.tagscombobox import TagsComboBoxEntry as Kind
@@ -35,4 +37,3 @@ class TTagsComboBoxEntry(TagsCombo):
     def test_custom(self):
         self.all.get_child().set_text("a new tag")
         self.failUnlessEqual(self.all.tag, "a new tag")
-add(TTagsComboBoxEntry)
